@@ -29,7 +29,12 @@
 
 #include "CSSPrimitiveValue.h"
 #include "CSSValueKeywords.h"
-#include "RenderStyle.h"
+#include "GraphicsTypes.h"
+#include "Path.h"
+#include "RenderStyleConstants.h"
+#include "SVGRenderStyleDefs.h"
+#include "TextDirection.h"
+#include "ThemeTypes.h"
 
 namespace WebCore {
 
@@ -161,148 +166,167 @@ template<> inline CSSPrimitiveValue::operator CompositeOperator() const
     }
 }
 
-template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EAppearance e)
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(ControlPart e)
     : m_type(CSS_IDENT)
 {
     switch (e) {
-        case NoAppearance:
+        case NoControlPart:
             m_value.ident = CSSValueNone;
             break;
-        case CheckboxAppearance:
+        case CheckboxPart:
             m_value.ident = CSSValueCheckbox;
             break;
-        case RadioAppearance:
+        case RadioPart:
             m_value.ident = CSSValueRadio;
             break;
-        case PushButtonAppearance:
+        case PushButtonPart:
             m_value.ident = CSSValuePushButton;
             break;
-        case SquareButtonAppearance:
+        case SquareButtonPart:
             m_value.ident = CSSValueSquareButton;
             break;
-        case ButtonAppearance:
+        case ButtonPart:
             m_value.ident = CSSValueButton;
             break;
-        case ButtonBevelAppearance:
+        case ButtonBevelPart:
             m_value.ident = CSSValueButtonBevel;
             break;
-        case DefaultButtonAppearance:
+        case DefaultButtonPart:
             m_value.ident = CSSValueDefaultButton;
             break;
-        case ListboxAppearance:
+        case ListboxPart:
             m_value.ident = CSSValueListbox;
             break;
-        case ListItemAppearance:
+        case ListItemPart:
             m_value.ident = CSSValueListitem;
             break;
-        case MediaFullscreenButtonAppearance:
+        case MediaFullscreenButtonPart:
             m_value.ident = CSSValueMediaFullscreenButton;
             break;
-        case MediaPlayButtonAppearance:
+        case MediaPlayButtonPart:
             m_value.ident = CSSValueMediaPlayButton;
             break;
-        case MediaMuteButtonAppearance:
+        case MediaMuteButtonPart:
             m_value.ident = CSSValueMediaMuteButton;
             break;
-        case MediaSeekBackButtonAppearance:
+        case MediaSeekBackButtonPart:
             m_value.ident = CSSValueMediaSeekBackButton;
             break;
-        case MediaSeekForwardButtonAppearance:
+        case MediaSeekForwardButtonPart:
             m_value.ident = CSSValueMediaSeekForwardButton;
             break;
-        case MediaSliderAppearance:
+        case MediaRewindButtonPart:
+            m_value.ident = CSSValueMediaRewindButton;
+            break;
+        case MediaReturnToRealtimeButtonPart:
+            m_value.ident = CSSValueMediaReturnToRealtimeButton;
+            break;
+        case MediaSliderPart:
             m_value.ident = CSSValueMediaSlider;
             break;
-        case MediaSliderThumbAppearance:
+        case MediaSliderThumbPart:
             m_value.ident = CSSValueMediaSliderthumb;
             break;
-        case MenulistAppearance:
+        case MediaControlsBackgroundPart:
+            m_value.ident = CSSValueMediaControlsBackground;
+            break;
+        case MediaCurrentTimePart:
+            m_value.ident = CSSValueMediaCurrentTimeDisplay;
+            break;
+        case MediaTimeRemainingPart:
+            m_value.ident = CSSValueMediaTimeRemainingDisplay;
+            break;
+        case MenulistPart:
             m_value.ident = CSSValueMenulist;
             break;
-        case MenulistButtonAppearance:
+        case MenulistButtonPart:
             m_value.ident = CSSValueMenulistButton;
             break;
-        case MenulistTextAppearance:
+        case MenulistTextPart:
             m_value.ident = CSSValueMenulistText;
             break;
-        case MenulistTextFieldAppearance:
+        case MenulistTextFieldPart:
             m_value.ident = CSSValueMenulistTextfield;
             break;
-        case ScrollbarButtonUpAppearance:
-            m_value.ident = CSSValueScrollbarbuttonUp;
-            break;
-        case ScrollbarButtonDownAppearance:
-            m_value.ident = CSSValueScrollbarbuttonDown;
-            break;
-        case ScrollbarButtonLeftAppearance:
-            m_value.ident = CSSValueScrollbarbuttonLeft;
-            break;
-        case ScrollbarButtonRightAppearance:
-            m_value.ident = CSSValueScrollbarbuttonRight;
-            break;
-        case ScrollbarTrackHorizontalAppearance:
-            m_value.ident = CSSValueScrollbartrackHorizontal;
-            break;
-        case ScrollbarTrackVerticalAppearance:
-            m_value.ident = CSSValueScrollbartrackVertical;
-            break;
-        case ScrollbarThumbHorizontalAppearance:
-            m_value.ident = CSSValueScrollbarthumbHorizontal;
-            break;
-        case ScrollbarThumbVerticalAppearance:
-            m_value.ident = CSSValueScrollbarthumbVertical;
-            break;
-        case ScrollbarGripperHorizontalAppearance:
-            m_value.ident = CSSValueScrollbargripperHorizontal;
-            break;
-        case ScrollbarGripperVerticalAppearance:
-            m_value.ident = CSSValueScrollbargripperVertical;
-            break;
-        case SliderHorizontalAppearance:
+        case SliderHorizontalPart:
             m_value.ident = CSSValueSliderHorizontal;
             break;
-        case SliderVerticalAppearance:
+        case SliderVerticalPart:
             m_value.ident = CSSValueSliderVertical;
             break;
-        case SliderThumbHorizontalAppearance:
+        case SliderThumbHorizontalPart:
             m_value.ident = CSSValueSliderthumbHorizontal;
             break;
-        case SliderThumbVerticalAppearance:
+        case SliderThumbVerticalPart:
             m_value.ident = CSSValueSliderthumbVertical;
             break;
-        case CaretAppearance:
+        case CaretPart:
             m_value.ident = CSSValueCaret;
             break;
-        case SearchFieldAppearance:
+        case SearchFieldPart:
             m_value.ident = CSSValueSearchfield;
             break;
-        case SearchFieldDecorationAppearance:
+        case SearchFieldDecorationPart:
             m_value.ident = CSSValueSearchfieldDecoration;
             break;
-        case SearchFieldResultsDecorationAppearance:
+        case SearchFieldResultsDecorationPart:
             m_value.ident = CSSValueSearchfieldResultsDecoration;
             break;
-        case SearchFieldResultsButtonAppearance:
+        case SearchFieldResultsButtonPart:
             m_value.ident = CSSValueSearchfieldResultsButton;
             break;
-        case SearchFieldCancelButtonAppearance:
+        case SearchFieldCancelButtonPart:
             m_value.ident = CSSValueSearchfieldCancelButton;
             break;
-        case TextFieldAppearance:
+        case TextFieldPart:
             m_value.ident = CSSValueTextfield;
             break;
-        case TextAreaAppearance:
+        case TextAreaPart:
             m_value.ident = CSSValueTextarea;
             break;
+        case CapsLockIndicatorPart:
+            m_value.ident = CSSValueCapsLockIndicator;
+            break;
     }
 }
 
-template<> inline CSSPrimitiveValue::operator EAppearance() const
+template<> inline CSSPrimitiveValue::operator ControlPart() const
 {
     if (m_value.ident == CSSValueNone)
-        return NoAppearance;
+        return NoControlPart;
     else
-        return EAppearance(m_value.ident - CSSValueCheckbox + 1);
+        return ControlPart(m_value.ident - CSSValueCheckbox + 1);
+}
+
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EFillAttachment e)
+    : m_type(CSS_IDENT)
+{
+    switch (e) {
+        case ScrollBackgroundAttachment:
+            m_value.ident = CSSValueScroll;
+            break;
+        case LocalBackgroundAttachment:
+            m_value.ident = CSSValueLocal;
+            break;
+        case FixedBackgroundAttachment:
+            m_value.ident = CSSValueFixed;
+            break;
+    }
+}
+
+template<> inline CSSPrimitiveValue::operator EFillAttachment() const
+{
+    switch (m_value.ident) {
+        case CSSValueScroll:
+            return ScrollBackgroundAttachment;
+        case CSSValueLocal:
+            return LocalBackgroundAttachment;
+        case CSSValueFixed:
+            return FixedBackgroundAttachment;
+        default:
+            ASSERT_NOT_REACHED();
+            return ScrollBackgroundAttachment;
+    }
 }
 
 template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EFillBox e)
@@ -310,13 +334,13 @@ template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EFillBox e)
 {
     switch (e) {
         case BorderFillBox:
-            m_value.ident = CSSValueBorder;
+            m_value.ident = CSSValueBorderBox;
             break;
         case PaddingFillBox:
-            m_value.ident = CSSValuePadding;
+            m_value.ident = CSSValuePaddingBox;
             break;
         case ContentFillBox:
-            m_value.ident = CSSValueContent;
+            m_value.ident = CSSValueContentBox;
             break;
         case TextFillBox:
             m_value.ident = CSSValueText;
@@ -328,10 +352,13 @@ template<> inline CSSPrimitiveValue::operator EFillBox() const
 {
     switch (m_value.ident) {
         case CSSValueBorder:
+        case CSSValueBorderBox:
             return BorderFillBox;
         case CSSValuePadding:
+        case CSSValuePaddingBox:
             return PaddingFillBox;
         case CSSValueContent:
+        case CSSValueContentBox:
             return ContentFillBox;
         case CSSValueText:
             return TextFillBox;
@@ -680,6 +707,12 @@ template<> inline CSSPrimitiveValue::CSSPrimitiveValue(ECursor e)
         case CURSOR_ALL_SCROLL:
             m_value.ident = CSSValueAllScroll;
             break;
+        case CURSOR_WEBKIT_GRAB:
+            m_value.ident = CSSValueWebkitGrab;
+            break;
+        case CURSOR_WEBKIT_GRABBING:
+            m_value.ident = CSSValueWebkitGrabbing;
+            break;
     }
 }
 
@@ -1683,6 +1716,72 @@ template<> inline CSSPrimitiveValue::operator TextDirection() const
     }
 }
 
+template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EPointerEvents e)
+    : m_type(CSS_IDENT)
+{
+    switch (e) {
+        case PE_NONE:
+            m_value.ident = CSSValueNone;
+            break;
+        case PE_STROKE:
+            m_value.ident = CSSValueStroke;
+            break;
+        case PE_FILL:
+            m_value.ident = CSSValueFill;
+            break;
+        case PE_PAINTED:
+            m_value.ident = CSSValuePainted;
+            break;
+        case PE_VISIBLE:
+            m_value.ident = CSSValueVisible;
+            break;
+        case PE_VISIBLE_STROKE:
+            m_value.ident = CSSValueVisiblestroke;
+            break;
+        case PE_VISIBLE_FILL:
+            m_value.ident = CSSValueVisiblefill;
+            break;
+        case PE_VISIBLE_PAINTED:
+            m_value.ident = CSSValueVisiblepainted;
+            break;
+        case PE_AUTO:
+            m_value.ident = CSSValueAuto;
+            break;
+        case PE_ALL:
+            m_value.ident = CSSValueAll;
+            break;
+    }
+}
+
+template<> inline CSSPrimitiveValue::operator EPointerEvents() const
+{
+    switch (m_value.ident) {
+        case CSSValueAll:
+            return PE_ALL;
+        case CSSValueAuto:
+            return PE_AUTO;
+        case CSSValueNone:
+            return PE_NONE;
+        case CSSValueVisiblepainted:
+            return PE_VISIBLE_PAINTED;
+        case CSSValueVisiblefill:
+            return PE_VISIBLE_FILL;
+        case CSSValueVisiblestroke:
+            return PE_VISIBLE_STROKE;
+        case CSSValueVisible:
+            return PE_VISIBLE;
+        case CSSValuePainted:
+            return PE_PAINTED;
+        case CSSValueFill:
+            return PE_FILL;
+        case CSSValueStroke:
+            return PE_STROKE;
+        default:
+            ASSERT_NOT_REACHED();
+            return PE_ALL;
+    }
+}
+
 #if ENABLE(SVG)
 
 template<> inline CSSPrimitiveValue::CSSPrimitiveValue(LineCap e)
@@ -2019,67 +2118,6 @@ template<> inline CSSPrimitiveValue::operator EImageRendering() const
     }
 }
 
-template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EPointerEvents e)
-    : m_type(CSS_IDENT)
-{
-    switch (e) {
-        case PE_NONE:
-            m_value.ident = CSSValueNone;
-            break;
-        case PE_STROKE:
-            m_value.ident = CSSValueStroke;
-            break;
-        case PE_FILL:
-            m_value.ident = CSSValueFill;
-            break;
-        case PE_PAINTED:
-            m_value.ident = CSSValuePainted;
-            break;
-        case PE_VISIBLE:
-            m_value.ident = CSSValueVisible;
-            break;
-        case PE_VISIBLE_STROKE:
-            m_value.ident = CSSValueVisiblestroke;
-            break;
-        case PE_VISIBLE_FILL:
-            m_value.ident = CSSValueVisiblefill;
-            break;
-        case PE_VISIBLE_PAINTED:
-            m_value.ident = CSSValueVisiblepainted;
-            break;
-        case PE_ALL:
-            m_value.ident = CSSValueAll;
-            break;
-    }
-}
-
-template<> inline CSSPrimitiveValue::operator EPointerEvents() const
-{
-    switch (m_value.ident) {
-        case CSSValueAll:
-            return PE_ALL;
-        case CSSValueNone:
-            return PE_NONE;
-        case CSSValueVisiblepainted:
-            return PE_VISIBLE_PAINTED;
-        case CSSValueVisiblefill:
-            return PE_VISIBLE_FILL;
-        case CSSValueVisiblestroke:
-            return PE_VISIBLE_STROKE;
-        case CSSValueVisible:
-            return PE_VISIBLE;
-        case CSSValuePainted:
-            return PE_PAINTED;
-        case CSSValueFill:
-            return PE_FILL;
-        case CSSValueStroke:
-            return PE_STROKE;
-        default:
-            ASSERT_NOT_REACHED();
-            return PE_ALL;
-    }
-}
-
 template<> inline CSSPrimitiveValue::CSSPrimitiveValue(EShapeRendering e)
     : m_type(CSS_IDENT)
 {
