@@ -1,6 +1,8 @@
 #!/usr/bin/perl -w
 
-# Copyright (C) 2005, 2006, 2007 Apple Inc. All rights reserved.
+# Copyright (C) 2005, 2006, 2007, 2009 Apple Inc. All rights reserved.
+# Copyright (C) 2009, Julien Chaffraix <jchaffraix@webkit.org>
+# Copyright (C) 2009 Torch Mobile Inc. All rights reserved. (http://www.torchmobile.com/)
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
@@ -32,9 +34,11 @@ use Config;
 use Getopt::Long;
 use File::Path;
 use IO::File;
+use InFilesParser;
 use Switch;
-use XMLTiny qw(parsefile);
 
+my $printFactory = 0; 
+my $printWrapperFactory = 0;
 my $tagsFile = "";
 my $attrsFile = "";
 my $outputDir = ".";
@@ -43,23 +47,22 @@ my %attrs = ();
 my %parameters = ();
 my $extraDefines = 0;
 my $preprocessor = "/usr/bin/gcc -E -P -x c++";
-my %svgCustomMappings = ();
-my %htmlCustomMappings = ();
 
 GetOptions('tags=s' => \$tagsFile, 
     'attrs=s' => \$attrsFile,
+    'factory' => \$printFactory,
     'outputDir=s' => \$outputDir,
     'extraDefines=s' => \$extraDefines,
-    'preprocessor=s' => \$preprocessor);
+    'preprocessor=s' => \$preprocessor,
+    'wrapperFactory' => \$printWrapperFactory);
 
 die "You must specify at least one of --tags <file> or --attrs <file>" unless (length($tagsFile) || length($attrsFile));
 
-readNames($tagsFile) if length($tagsFile);
-readNames($attrsFile) if length($attrsFile);
+readNames($tagsFile, "tags") if length($tagsFile);
+readNames($attrsFile, "attrs") if length($attrsFile);
 
 die "You must specify a namespace (e.g. SVG) for <namespace>Names.h" unless $parameters{'namespace'};
 die "You must specify a namespaceURI (e.g. http://www.w3.org/2000/svg)" unless $parameters{'namespaceURI'};
-die "You must specify a cppNamespace (e.g. DOM) used for <cppNamespace>::<namespace>Names::fooTag" unless $parameters{'cppNamespace'};
 
 $parameters{'namespacePrefix'} = $parameters{'namespace'} unless $parameters{'namespacePrefix'};
 
@@ -71,12 +74,12 @@ my $wrapperFactoryBasePath = "$outputDir/JS$parameters{'namespace'}ElementWrappe
 printNamesHeaderFile("$namesBasePath.h");
 printNamesCppFile("$namesBasePath.cpp");
 
-if ($parameters{'generateFactory'}) {
+if ($printFactory) {
     printFactoryCppFile("$factoryBasePath.cpp");
     printFactoryHeaderFile("$factoryBasePath.h");
 }
 
-if ($parameters{'generateWrapperFactory'}) {
+if ($printWrapperFactory) {
     printWrapperFactoryCppFile("$wrapperFactoryBasePath.cpp");
     printWrapperFactoryHeaderFile("$wrapperFactoryBasePath.h");
 }
@@ -85,13 +88,20 @@ if ($parameters{'generateWrapperFactory'}) {
 
 sub initializeTagPropertyHash
 {
-    return ('upperCase' => upperCaseName($_[0]),
-            'applyAudioHack' => 0);
+    return ('constructorNeedsCreatedByParser' => 0,
+            'constructorNeedsFormElement' => 0,
+            'exportString' => 0,
+            'interfaceName' => defaultInterfaceName($_[0]),
+            # By default, the JSInterfaceName is the same as the interfaceName.
+            'JSInterfaceName' => defaultInterfaceName($_[0]),
+            'mapToTagName' => '',
+            'wrapperOnlyIfMediaIsAvailable' => 0,
+            'conditional' => 0);
 }
 
 sub initializeAttrPropertyHash
 {
-    return ('upperCase' => upperCaseName($_[0]));
+    return ('exportString' => 0);
 }
 
 sub initializeParametersHash
@@ -99,93 +109,74 @@ sub initializeParametersHash
     return ('namespace' => '',
             'namespacePrefix' => '',
             'namespaceURI' => '',
-            'cppNamespace' => '',
-            'generateFactory' => 0,
             'guardFactoryWith' => '',
-            'generateWrapperFactory' => 0,
-            # The 2 nullNamespace properties are generated from the "nullNamespace" attribute with respect to the file parsed (attrs or tags).
             'tagsNullNamespace' => 0,
-            'attrsNullNamespace' => 0);
+            'attrsNullNamespace' => 0,
+            'exportStrings' => 0);
+}
+
+sub defaultInterfaceName
+{
+    die "No namespace found" if !$parameters{'namespace'};
+    return $parameters{'namespace'} . upperCaseName($_[0]) . "Element"
 }
 
 ### Parsing handlers
 
-# Our files should have the following form :
-# <'tags' or 'attrs' globalProperty1 = 'value1' ... />
-# <'tag/attr name' 'property1' = 'value1' ... />
-# where the properties are defined in the initialize*PropertyHash methods.
-# (more tag/attr ...)
-# </tags> or </attrs>
-
-sub parseTags
+sub tagsHandler
 {
-    my $contentsRef = shift;
-    foreach my $contentRef (@$contentsRef) {
-        my $tag = $${contentRef}{'name'};
-        $tag =~ s/-/_/g;
+    my ($tag, $property, $value) = @_;
 
-        # Initialize default properties' values.
-        $tags{$tag} = { initializeTagPropertyHash($tag) } if !defined($tags{$tag});
+    $tag =~ s/-/_/g;
 
-        # Parse the XML attributes.
-        my %properties = %{$$contentRef{'attrib'}};
-        foreach my $property (keys %properties) {
-            die "Unknown property $property for tag $tag\n" if !defined($tags{$tag}{$property});
-            $tags{$tag}{$property} = $properties{$property};
+    # Initialize default properties' values.
+    $tags{$tag} = { initializeTagPropertyHash($tag) } if !defined($tags{$tag});
+
+    if ($property) {
+        die "Unknown property $property for tag $tag\n" if !defined($tags{$tag}{$property});
+        # The code rely on JSInterfaceName deriving from interfaceName to check for custom JSInterfaceName.
+        # So just override JSInterfaceName if it was not already set.
+        if ($property eq "interfaceName" && $tags{$tag}{'JSInterfaceName'} eq $tags{$tag}{'interfaceName'}) {
+                $tags{$tag}{'JSInterfaceName'} = $value;
         }
+        $tags{$tag}{$property} = $value;
     }
 }
 
-sub parseAttrs
+sub attrsHandler
 {
-    my $contentsRef = shift;
-    foreach my $contentRef (@$contentsRef) {
-        my $attr = $${contentRef}{'name'};
-        $attr =~ s/-/_/g;
+    my ($attr, $property, $value) = @_;
 
-        # Initialize default properties' values.
-        $attrs{$attr} = { initializeAttrPropertyHash($attr) } if !defined($attrs{$attr});
+    $attr =~ s/-/_/g;
 
-        # Parse the XML attributes.
-        my %properties = %{$$contentRef{'attrib'}};
-        foreach my $property (keys %properties) {
-            die "Unknown property $property for attribute $attr\n" if !defined($attrs{$attr}{$property});
-            $attrs{$attr}{$property} = $properties{$property};
-        }
+    # Initialize default properties' values.
+    $attrs{$attr} = { initializeAttrPropertyHash($attr) } if !defined($attrs{$attr});
+
+    if ($property) {
+        die "Unknown property $property for attribute $attr\n" if !defined($attrs{$attr}{$property});
+        $attrs{$attr}{$property} = $value;
     }
 }
 
-sub parseParameters
+sub parametersHandler
 {
-    my ($propertiesRef, $elementName) = @_;
-    my %properties = %$propertiesRef;
+    my ($parameter, $value) = @_;
 
     # Initialize default properties' values.
     %parameters = initializeParametersHash() if !(keys %parameters);
 
-    # Parse the XML attributes.
-    foreach my $property (keys %properties) {
-        # This is used in case we want to change the parameter name depending
-        # on what is parsed.
-        my $parameter = $property;
-
-        # "nullNamespace" case
-        if ($property eq "nullNamespace") {
-            $parameter = $elementName.(ucfirst $property);
-        }
-
-        die "Unknown parameter $property for tags/attrs\n" if !defined($parameters{$parameter});
-        $parameters{$parameter} = $properties{$property};
-    }
+    die "Unknown parameter $parameter for tags/attrs\n" if !defined($parameters{$parameter});
+    $parameters{$parameter} = $value;
 }
 
 ## Support routines
 
 sub readNames
 {
-    my $namesFile = shift;
+    my ($namesFile, $type) = @_;
 
     my $names = new IO::File;
+
     if ($extraDefines eq 0) {
         open($names, $preprocessor . " " . $namesFile . "|") or die "Failed to open file: $namesFile";
     } else {
@@ -196,23 +187,17 @@ sub readNames
     my $tagsCount = keys %tags;
     my $attrsCount = keys %attrs;
 
-    my $documentRef = parsefile($names);
+    my $InParser = InFilesParser->new();
 
-    # XML::Tiny returns an array reference to a hash containing the different properties
-    my %document = %{@$documentRef[0]};
-    my $name = $document{'name'};
-
-    # Check root element to determine what we are parsing
-    switch($name) {
+    switch ($type) {
         case "tags" {
-            parseParameters(\%{$document{'attrib'}}, $name);
-            parseTags(\@{$document{'content'}});
+            $InParser->parse($names, \&parametersHandler, \&tagsHandler);
         }
         case "attrs" {
-            parseParameters(\%{$document{'attrib'}}, $name);
-            parseAttrs(\@{$document{'content'}});
-        } else {
-            die "Do not know how to parse file starting with $name!\n";
+            $InParser->parse($names, \&parametersHandler, \&attrsHandler);
+        }
+        else {
+            die "Do not know how to parse $type";
         }
     }
 
@@ -224,33 +209,158 @@ sub readNames
 sub printMacros
 {
     my ($F, $macro, $suffix, $namesRef) = @_;
+    my %names = %$namesRef;
+
     for my $name (sort keys %$namesRef) {
-        print F "    $macro $name","$suffix;\n";
+        print F "$macro $name","$suffix;\n";
+
+        if ($parameters{'exportStrings'} or $names{$name}{"exportString"}) { 
+            print F "extern char $name", "${suffix}String[];\n";
+        }
     }
 }
 
+sub usesDefaultWrapper
+{
+    my $tagName = shift;
+    return $tagName eq $parameters{'namespace'} . "Element";
+}
+
+# Build a direct mapping from the tags to the Element to create, excluding
+# Element that have not constructor.
+sub buildConstructorMap
+{
+    my %tagConstructorMap = ();
+    for my $tagName (keys %tags) {
+        my $interfaceName = $tags{$tagName}{'interfaceName'};
+        next if (usesDefaultWrapper($interfaceName));
+
+        if ($tags{$tagName}{'mapToTagName'}) {
+            die "Cannot handle multiple mapToTagName for $tagName\n" if $tags{$tags{$tagName}{'mapToTagName'}}{'mapToTagName'};
+            $interfaceName = $tags{ $tags{$tagName}{'mapToTagName'} }{'interfaceName'};
+        }
+
+        # Chop the string to keep the interesting part.
+        $interfaceName =~ s/$parameters{'namespace'}(.*)Element/$1/;
+        $tagConstructorMap{$tagName} = lc($interfaceName);
+    }
+
+    return %tagConstructorMap;
+}
+
+# Helper method that print the constructor's signature avoiding
+# unneeded arguments.
+sub printConstructorSignature
+{
+    my ($F, $tagName, $constructorName, $constructorTagName) = @_;
+
+    print F "static PassRefPtr<$parameters{'namespace'}Element> ${constructorName}Constructor(const QualifiedName& $constructorTagName, Document* doc";
+    if ($parameters{'namespace'} eq "HTML") {
+        print F ", HTMLFormElement*";
+        if ($tags{$tagName}{'constructorNeedsFormElement'}) {
+            print F " formElement";
+        }
+    }
+    print F ", bool";
+    if ($tags{$tagName}{'constructorNeedsCreatedByParser'}) {
+        print F " createdByParser"; 
+    }
+    print F ")\n{\n";
+}
+
+# Helper method to dump the constructor interior and call the 
+# Element constructor with the right arguments.
+# The variable names should be kept in sync with the previous method.
+sub printConstructorInterior
+{
+    my ($F, $tagName, $interfaceName, $constructorTagName) = @_;
+
+    # Handle media elements.
+    if ($tags{$tagName}{'wrapperOnlyIfMediaIsAvailable'}) {
+        print F <<END
+    if (!MediaPlayer::isAvailable())
+        return new HTMLElement($constructorTagName, doc);
+END
+;
+    }
+
+    # Now call the constructor with the right parameters.
+    print F "    return new ${interfaceName}($constructorTagName, doc";
+    if ($tags{$tagName}{'constructorNeedsFormElement'}) {
+        print F ", formElement";
+    }
+    if ($tags{$tagName}{'constructorNeedsCreatedByParser'}) {
+        print F ", createdByParser";
+    }
+    print F ");\n}\n\n";
+}
+
 sub printConstructors
 {
-    my ($F, $namesRef) = @_;
-    my %names = %$namesRef;
+    my ($F, $tagConstructorMapRef) = @_;
+    my %tagConstructorMap = %$tagConstructorMapRef;
 
     print F "#if $parameters{'guardFactoryWith'}\n" if $parameters{'guardFactoryWith'};
-    for my $name (sort keys %names) {
-        my $ucName = $names{$name}{"upperCase"};
 
-        print F "$parameters{'namespace'}Element* ${name}Constructor(Document* doc, bool createdByParser)\n";
-        print F "{\n";
-        print F "    return new $parameters{'namespace'}${ucName}Element(${name}Tag, doc);\n";
-        print F "}\n\n";
+    # This is to avoid generating the same constructor several times.
+    my %uniqueTags = ();
+    for my $tagName (sort keys %tagConstructorMap) {
+        my $interfaceName = $tags{$tagName}{'interfaceName'};
+
+        # Ignore the mapped tag
+        # FIXME: It could be moved inside this loop but was split for readibility.
+        next if (defined($uniqueTags{$interfaceName}) || $tags{$tagName}{'mapToTagName'});
+
+        $uniqueTags{$interfaceName} = '1';
+
+        my $conditional = $tags{$tagName}{"conditional"};
+        if ($conditional) {
+            my $conditionalString = "ENABLE(" . join(") && ENABLE(", split(/&/, $conditional)) . ")";
+            print F "#if ${conditionalString}\n\n";
+        }
+
+        printConstructorSignature($F, $tagName, $tagConstructorMap{$tagName}, "tagName");
+        printConstructorInterior($F, $tagName, $interfaceName, "tagName");
+
+        if ($conditional) {
+            print F "#endif\n\n";
+        }
     }
+
+    # Mapped tag name uses a special wrapper to keep their prefix and namespaceURI while using the mapped localname.
+    for my $tagName (sort keys %tagConstructorMap) {
+        if ($tags{$tagName}{'mapToTagName'}) {
+            my $mappedName = $tags{$tagName}{'mapToTagName'};
+            printConstructorSignature($F, $mappedName, $mappedName . "To" . $tagName, "tagName");
+            printConstructorInterior($F, $mappedName, $tags{$mappedName}{'interfaceName'}, "QualifiedName(tagName.prefix(), ${mappedName}Tag.localName(), tagName.namespaceURI())");
+        }
+    }
+
     print F "#endif\n" if $parameters{'guardFactoryWith'};
 }
 
 sub printFunctionInits
 {
-    my ($F, $namesRef) = @_;
-    for my $name (sort keys %$namesRef) {
-        print F "    gFunctionMap->set(${name}Tag.localName().impl(), ${name}Constructor);\n";
+    my ($F, $tagConstructorMap) = @_;
+    my %tagConstructorMap = %$tagConstructorMap;
+
+    for my $tagName (sort keys %tagConstructorMap) {
+
+        my $conditional = $tags{$tagName}{"conditional"};
+        if ($conditional) {
+            my $conditionalString = "ENABLE(" . join(") && ENABLE(", split(/&/, $conditional)) . ")";
+            print F "#if ${conditionalString}\n";
+        }
+
+        if ($tags{$tagName}{'mapToTagName'}) {
+            print F "    addTag(${tagName}Tag, $tags{$tagName}{'mapToTagName'}To${tagName}Constructor);\n";
+        } else {
+            print F "    addTag(${tagName}Tag, $tagConstructorMap{$tagName}Constructor);\n";
+        }
+
+        if ($conditional) {
+            print F "#endif\n\n";
+        }
     }
 }
 
@@ -324,7 +434,7 @@ sub printNamesHeaderFile
     print F "#define DOM_$parameters{'namespace'}NAMES_H\n\n";
     print F "#include \"QualifiedName.h\"\n\n";
     
-    print F "namespace $parameters{'cppNamespace'} { namespace $parameters{'namespace'}Names {\n\n";
+    print F "namespace WebCore {\n\n namespace $parameters{'namespace'}Names {\n\n";
     
     my $lowerNamespace = lc($parameters{'namespacePrefix'});
     print F "#ifndef DOM_$parameters{'namespace'}NAMES_HIDE_GLOBALS\n";
@@ -334,16 +444,23 @@ sub printNamesHeaderFile
     if (keys %tags) {
         print F "// Tags\n";
         printMacros($F, "extern const WebCore::QualifiedName", "Tag", \%tags);
-        print F "\n\nWebCore::QualifiedName** get$parameters{'namespace'}Tags(size_t* size);\n";
     }
     
     if (keys %attrs) {
         print F "// Attributes\n";
         printMacros($F, "extern const WebCore::QualifiedName", "Attr", \%attrs);
-        print F "\n\nWebCore::QualifiedName** get$parameters{'namespace'}Attr(size_t* size);\n";
     }
     print F "#endif\n\n";
-    print F "void init();\n\n";
+    
+    if (keys %tags) {
+        print F "WebCore::QualifiedName** get$parameters{'namespace'}Tags(size_t* size);\n";
+    }
+
+    if (keys %attrs) {
+        print F "WebCore::QualifiedName** get$parameters{'namespace'}Attrs(size_t* size);\n";
+    }
+    
+    print F "\nvoid init();\n\n";
     print F "} }\n\n";
     print F "#endif\n\n";
     
@@ -362,7 +479,7 @@ sub printNamesCppFile
 
 print F "#include \"config.h\"\n";
 
-print F "#ifdef AVOID_STATIC_CONSTRUCTORS\n";
+print F "#ifdef SKIP_STATIC_CONSTRUCTORS_ON_GCC\n";
 print F "#define DOM_$parameters{'namespace'}NAMES_HIDE_GLOBALS 1\n";
 print F "#else\n";
 print F "#define QNAME_DEFAULT_CONSTRUCTOR 1\n";
@@ -372,7 +489,7 @@ print F "#endif\n\n";
 print F "#include \"$parameters{'namespace'}Names.h\"\n\n";
 print F "#include \"StaticConstructors.h\"\n";
 
-print F "namespace $parameters{'cppNamespace'} { namespace $parameters{'namespace'}Names {
+print F "namespace WebCore {\n\n namespace $parameters{'namespace'}Names {
 
 using namespace WebCore;
 
@@ -394,7 +511,6 @@ DEFINE_GLOBAL(AtomicString, ${lowerNamespace}NamespaceURI, \"$parameters{'namesp
         print F "    *size = ", scalar(keys %tags), ";\n";
         print F "    return $parameters{'namespace'}Tags;\n";
         print F "}\n";
-        
     }
 
     if (keys %attrs) {
@@ -413,6 +529,14 @@ DEFINE_GLOBAL(AtomicString, ${lowerNamespace}NamespaceURI, \"$parameters{'namesp
         print F "}\n";
     }
 
+    if (keys %tags) {
+        printDefinitionStrings($F, \%tags, "tags");
+    }
+
+    if (keys %attrs) {
+        printDefinitionStrings($F, \%attrs, "attributes");
+    }
+
 print F "\nvoid init()
 {
     static bool initialized = false;
@@ -444,28 +568,51 @@ print F "\nvoid init()
 
 sub printJSElementIncludes
 {
-    my ($F, $namesRef) = @_;
-    my %names = %$namesRef;
-    for my $name (sort keys %names) {
-        next if (hasCustomMapping($name));
+    my $F = shift;
 
-        my $ucName = $names{$name}{"upperCase"};
-        print F "#include \"JS$parameters{'namespace'}${ucName}Element.h\"\n";
+    my %tagsSeen;
+    for my $tagName (sort keys %tags) {
+        my $JSInterfaceName = $tags{$tagName}{"JSInterfaceName"};
+        next if defined($tagsSeen{$JSInterfaceName}) || usesDefaultJSWrapper($tagName);
+        $tagsSeen{$JSInterfaceName} = 1;
+
+        print F "#include \"JS${JSInterfaceName}.h\"\n";
     }
 }
 
 sub printElementIncludes
 {
-    my ($F, $namesRef, $shouldSkipCustomMappings) = @_;
-    my %names = %$namesRef;
-    for my $name (sort keys %names) {
-        next if ($shouldSkipCustomMappings && hasCustomMapping($name));
+    my $F = shift;
 
-        my $ucName = $names{$name}{"upperCase"};
-        print F "#include \"$parameters{'namespace'}${ucName}Element.h\"\n";
+    my %tagsSeen;
+    for my $tagName (sort keys %tags) {
+        my $interfaceName = $tags{$tagName}{"interfaceName"};
+        next if defined($tagsSeen{$interfaceName});
+        $tagsSeen{$interfaceName} = 1;
+
+        print F "#include \"${interfaceName}.h\"\n";
     }
 }
 
+sub printDefinitionStrings
+{
+    my ($F, $namesRef, $type) = @_;
+    my $singularType = substr($type, 0, -1);
+    my $shortType = substr($singularType, 0, 4);
+    my $shortCamelType = ucfirst($shortType);
+    print F "\n// " . ucfirst($type) . " as strings\n";
+
+    my %names = %$namesRef;
+    for my $name (sort keys %$namesRef) {
+        next if (!$parameters{'exportStrings'} and !$names{$name}{"exportString"});
+
+        my $realName = $name;
+        $realName =~ s/_/-/g;
+
+        print F "char $name","${shortCamelType}String[] = \"$realName\";\n";
+    }
+} 
+
 sub printDefinitions
 {
     my ($F, $namesRef, $type, $namespaceURI) = @_;
@@ -476,23 +623,20 @@ sub printDefinitions
     
     print F "    // " . ucfirst($type) . "\n";
 
+    my %names = %$namesRef;
     for my $name (sort keys %$namesRef) {
-        print F "    const char *$name","${shortCamelType}String = \"$name\";\n";
+        next if ($parameters{'exportStrings'} or $names{$name}{"exportString"});
+
+        my $realName = $name;
+        $realName =~ s/_/-/g;
+        print F "    const char *$name","${shortCamelType}String = \"$realName\";\n";
     }
-        
-    for my $name (sort keys %$namesRef) {
-        if ($name =~ /_/) {
-            my $realName = $name;
-            $realName =~ s/_/-/g;
-            print F "    ${name}${shortCamelType}String = \"$realName\";\n";
-        }
-    }
-    print F "\n";
+
+    print "\n";
 
     for my $name (sort keys %$namesRef) {
         print F "    new ((void*)&$name","${shortCamelType}) QualifiedName(nullAtom, $name","${shortCamelType}String, $namespaceURI);\n";
     }
-
 }
 
 ## ElementFactory routines
@@ -508,35 +652,59 @@ printLicenseHeader($F);
 print F <<END
 #include "config.h"
 #include "$parameters{'namespace'}ElementFactory.h"
+
 #include "$parameters{'namespace'}Names.h"
-#include "Page.h"
+#if ENABLE(DASHBOARD_SUPPORT)
+#include "Document.h"
 #include "Settings.h"
+#endif
+
 END
 ;
 
-printElementIncludes($F, \%tags, 0);
+if ($parameters{'namespace'} eq "HTML") {
+    print F "#include \"HTMLFormElement.h\"\n";
+}
+
+printElementIncludes($F);
 
 print F <<END
 #include <wtf/HashMap.h>
 
-using namespace WebCore;
-using namespace $parameters{'cppNamespace'}::$parameters{'namespace'}Names;
+namespace WebCore {
 
-typedef $parameters{'namespace'}Element* (*ConstructorFunc)(Document* doc, bool createdByParser);
-typedef WTF::HashMap<AtomicStringImpl*, ConstructorFunc> FunctionMap;
-
-static FunctionMap* gFunctionMap = 0;
-
-namespace $parameters{'cppNamespace'} {
+using namespace $parameters{'namespace'}Names;
 
 END
 ;
 
-printConstructors($F, \%tags);
+print F "typedef PassRefPtr<$parameters{'namespace'}Element> (*ConstructorFunction)(const QualifiedName&, Document*";
+
+if ($parameters{'namespace'} eq "HTML") {
+    print F ", HTMLFormElement*";
+}
+
+print F ", bool createdByParser);\n";
+print F <<END
+typedef HashMap<AtomicStringImpl*, ConstructorFunction> FunctionMap;
+
+static FunctionMap* gFunctionMap = 0;
+
+END
+;
+
+my %tagConstructorMap = buildConstructorMap();
+
+printConstructors($F, \%tagConstructorMap);
 
 print F "#if $parameters{'guardFactoryWith'}\n" if $parameters{'guardFactoryWith'};
 
 print F <<END
+static void addTag(const QualifiedName& tag, ConstructorFunction func)
+{
+    gFunctionMap->set(tag.localName().impl(), func);
+}
+
 static inline void createFunctionMapIfNecessary()
 {
     if (gFunctionMap)
@@ -548,16 +716,18 @@ static inline void createFunctionMapIfNecessary()
 END
 ;
 
-printFunctionInits($F, \%tags);
+printFunctionInits($F, \%tagConstructorMap);
 
 print F "}\n";
-print F "#endif\n\n" if $parameters{'guardFactoryWith'};
+print F "#endif\n" if $parameters{'guardFactoryWith'};
 
-print F <<END
-$parameters{'namespace'}Element* $parameters{'namespace'}ElementFactory::create$parameters{'namespace'}Element(const QualifiedName& qName, Document* doc, bool createdByParser)
-{
-END
-;
+print F "\nPassRefPtr<$parameters{'namespace'}Element> $parameters{'namespace'}ElementFactory::create$parameters{'namespace'}Element(const QualifiedName& qName, Document* doc";
+
+if ($parameters{"namespace"} eq "HTML") {
+    print F ", HTMLFormElement* formElement";
+}
+
+print F ", bool createdByParser)\n{\n";
 
 print F "#if $parameters{'guardFactoryWith'}\n" if $parameters{'guardFactoryWith'};
 
@@ -566,21 +736,36 @@ print F <<END
     if (!doc)
         return 0;
 
+END
+;
+
+if ($parameters{'namespace'} ne "HTML") {
+print F <<END
 #if ENABLE(DASHBOARD_SUPPORT)
     Settings* settings = doc->settings();
     if (settings && settings->usesDashboardBackwardCompatibilityMode())
         return 0;
 #endif
-
-    createFunctionMapIfNecessary();
-    ConstructorFunc func = gFunctionMap->get(qName.localName().impl());
-    if (func)
-        return func(doc, createdByParser);
-
-    return new $parameters{'namespace'}Element(qName, doc);
 END
 ;
 
+}
+
+print F <<END
+    createFunctionMapIfNecessary();
+    ConstructorFunction func = gFunctionMap->get(qName.localName().impl());
+    if (func)
+END
+;
+
+if ($parameters{"namespace"} eq "HTML") {
+    print F "        return func(qName, doc, formElement, createdByParser);\n";
+} else {
+    print F "        return func(qName, doc, createdByParser);\n";
+}
+
+print F "    return new $parameters{'namespace'}Element(qName, doc);\n";
+
 if ($parameters{'guardFactoryWith'}) {
 
 print F <<END
@@ -595,7 +780,7 @@ END
 print F <<END
 }
 
-} // namespace
+} // namespace WebCore
 
 END
 ;
@@ -611,10 +796,12 @@ sub printFactoryHeaderFile
 
     printLicenseHeader($F);
 
-print F "#ifndef $parameters{'namespace'}ELEMENTFACTORY_H\n";
-print F "#define $parameters{'namespace'}ELEMENTFACTORY_H\n\n";
+    print F<<END
+#ifndef $parameters{'namespace'}ElementFactory_h
+#define $parameters{'namespace'}ElementFactory_h
+
+#include <wtf/PassRefPtr.h>
 
-print F "
 namespace WebCore {
     class Element;
     class Document;
@@ -622,129 +809,97 @@ namespace WebCore {
     class AtomicString;
 }
 
-namespace $parameters{'cppNamespace'}
-{
-    class $parameters{'namespace'}Element;
+namespace WebCore {
 
+    class $parameters{'namespace'}Element;
+END
+;
+
+if ($parameters{'namespace'} eq "HTML") {
+    print F "     class HTMLFormElement;\n";
+}
+
+print F<<END
     // The idea behind this class is that there will eventually be a mapping from namespace URIs to ElementFactories that can dispense
-    // elements.  In a compound document world, the generic createElement function (will end up being virtual) will be called.
-    class $parameters{'namespace'}ElementFactory
-    {
+    // elements. In a compound document world, the generic createElement function (will end up being virtual) will be called.
+    class $parameters{'namespace'}ElementFactory {
     public:
-        WebCore::Element* createElement(const WebCore::QualifiedName& qName, WebCore::Document* doc, bool createdByParser = true);
-        static $parameters{'namespace'}Element* create$parameters{'namespace'}Element(const WebCore::QualifiedName& qName, WebCore::Document* doc, bool createdByParser = true);
+        PassRefPtr<Element> createElement(const WebCore::QualifiedName&, WebCore::Document*, bool createdByParser = true);
+END
+;
+print F "        static PassRefPtr<$parameters{'namespace'}Element> create$parameters{'namespace'}Element(const WebCore::QualifiedName&, WebCore::Document*";
+
+if ($parameters{'namespace'} eq "HTML") {
+    print F ", HTMLFormElement* = 0";
+}
+
+print F ", bool /*createdByParser*/ = true);\n";
+
+printf F<<END
     };
 }
 
-#endif
+#endif // $parameters{'namespace'}ElementFactory_h
 
-";
+END
+;
 
     close F;
 }
 
 ## Wrapper Factory routines
 
-sub initializeCustomMappings
-{
-    if (!keys %svgCustomMappings) {
-        # These are used to map a tag to another one in WrapperFactory
-        # (for example, "h2" is mapped to "h1" so that they use the same JS Wrapper ("h1" wrapper))
-        # Mapping to an empty string will not generate a wrapper
-        %svgCustomMappings = ('animateMotion' => '',
-                              'hkern' => '',
-                              'mpath' => '');
-        %htmlCustomMappings = ('abbr' => '',
-                               'acronym' => '',
-                               'address' => '',
-                               'b' => '',
-                               'bdo' => '',
-                               'big' => '',
-                               'center' => '',
-                               'cite' => '',
-                               'code' => '',
-                               'colgroup' => 'col',
-                               'dd' => '',
-                               'dfn' => '',
-                               'dt' => '',
-                               'em' => '',
-                               'h2' => 'h1',
-                               'h3' => 'h1',
-                               'h4' => 'h1',
-                               'h5' => 'h1',
-                               'h6' => 'h1',
-                               'i' => '',
-                               'image' => 'img',
-                               'ins' => 'del',
-                               'kbd' => '',
-                               'keygen' => 'select',
-                               'listing' => 'pre',
-                               'layer' => '',
-                               'nobr' => '',
-                               'noembed' => '',
-                               'noframes' => '',
-                               'nolayer' => '',
-                               'noscript' => '',
-                               'plaintext' => '',
-                               's' => '',
-                               'samp' => '',
-                               'small' => '',
-                               'span' => '',
-                               'strike' => '',
-                               'strong' => '',
-                               'sub' => '',
-                               'sup' => '',
-                               'tfoot' => 'tbody',
-                               'th' => 'td',
-                               'thead' => 'tbody',
-                               'tt' => '',
-                               'u' => '',
-                               'var' => '',
-                               'wbr' => '',
-                               'xmp' => 'pre');
-    }
-}
-
-sub hasCustomMapping
+sub usesDefaultJSWrapper
 {
     my $name = shift;
-    initializeCustomMappings();
-    return 1 if $parameters{'namespace'} eq "HTML" && exists($htmlCustomMappings{$name});
-    return 1 if $parameters{'namespace'} eq "SVG" && exists($svgCustomMappings{$name});
-    return 0;
+
+    # A tag reuses the default wrapper if its JSInterfaceName matches the default namespace Element.
+    return $tags{$name}{'JSInterfaceName'} eq $parameters{"namespace"} . "Element" || $tags{$name}{'JSInterfaceName'} eq "HTMLNoScriptElement";
 }
 
 sub printWrapperFunctions
 {
-    my ($F, $namesRef) = @_;
-    my %names = %$namesRef;
-    for my $name (sort keys %names) {
-        # Custom mapping do not need a JS wrapper
-        next if (hasCustomMapping($name));
+    my $F = shift;
+
+    my %tagsSeen;
+    for my $tagName (sort keys %tags) {
+        # Avoid defining the same wrapper method twice.
+        my $JSInterfaceName = $tags{$tagName}{"JSInterfaceName"};
+        next if defined($tagsSeen{$JSInterfaceName}) || usesDefaultJSWrapper($tagName);
+        $tagsSeen{$JSInterfaceName} = 1;
+
+        my $conditional = $tags{$tagName}{"conditional"};
+        if ($conditional) {
+            my $conditionalString = "ENABLE(" . join(") && ENABLE(", split(/&/, $conditional)) . ")";
+            print F "#if ${conditionalString}\n\n";
+        }
 
-        my $ucName = $names{$name}{"upperCase"};
         # Hack for the media tags
-        if ($names{$name}{"applyAudioHack"}) {
+        # FIXME: This should have been done via a CustomWrapper attribute and a separate *Custom file.
+        if ($tags{$tagName}{"wrapperOnlyIfMediaIsAvailable"}) {
             print F <<END
-static JSNode* create${ucName}Wrapper(ExecState* exec, PassRefPtr<$parameters{'namespace'}Element> element)
+static JSNode* create${JSInterfaceName}Wrapper(ExecState* exec, JSDOMGlobalObject* globalObject, PassRefPtr<$parameters{'namespace'}Element> element)
 {
     if (!MediaPlayer::isAvailable())
-        return new JS$parameters{'namespace'}Element(JS$parameters{'namespace'}ElementPrototype::self(exec), element.get());
-    return new JS$parameters{'namespace'}${ucName}Element(JS$parameters{'namespace'}${ucName}ElementPrototype::self(exec), static_cast<$parameters{'namespace'}${ucName}Element*>(element.get()));
+        return CREATE_DOM_NODE_WRAPPER(exec, globalObject, $parameters{'namespace'}Element, element.get());
+    return CREATE_DOM_NODE_WRAPPER(exec, globalObject, ${JSInterfaceName}, element.get());
 }
 
 END
 ;
         } else {
             print F <<END
-static JSNode* create${ucName}Wrapper(ExecState* exec, PassRefPtr<$parameters{'namespace'}Element> element)
-{   
-    return new JS$parameters{'namespace'}${ucName}Element(JS$parameters{'namespace'}${ucName}ElementPrototype::self(exec), static_cast<$parameters{'namespace'}${ucName}Element*>(element.get()));
+static JSNode* create${JSInterfaceName}Wrapper(ExecState* exec, JSDOMGlobalObject* globalObject, PassRefPtr<$parameters{'namespace'}Element> element)
+{
+    return CREATE_DOM_NODE_WRAPPER(exec, globalObject, ${JSInterfaceName}, element.get());
 }
 
 END
 ;
         }
+        if ($conditional) {
+            print F "#endif\n\n";
+        }
     }
 }
 
@@ -762,58 +917,61 @@ sub printWrapperFactoryCppFile
 
     print F "#include \"JS$parameters{'namespace'}ElementWrapperFactory.h\"\n";
 
-    printJSElementIncludes($F, \%tags);
+    printJSElementIncludes($F);
 
     print F "\n#include \"$parameters{'namespace'}Names.h\"\n\n";
 
-    printElementIncludes($F, \%tags, 1);
+    printElementIncludes($F);
 
+    print F "\n#include <wtf/StdLibExtras.h>\n\n";
+    
     print F <<END
-using namespace KJS;
+using namespace JSC;
 
 namespace WebCore {
 
 using namespace $parameters{'namespace'}Names;
 
-typedef JSNode* (*Create$parameters{'namespace'}ElementWrapperFunction)(ExecState*, PassRefPtr<$parameters{'namespace'}Element>);
+typedef JSNode* (*Create$parameters{'namespace'}ElementWrapperFunction)(ExecState*, JSDOMGlobalObject*, PassRefPtr<$parameters{'namespace'}Element>);
 
 END
 ;
 
-    printWrapperFunctions($F, \%tags);
+    printWrapperFunctions($F);
 
     print F <<END
-JSNode* createJS$parameters{'namespace'}Wrapper(ExecState* exec, PassRefPtr<$parameters{'namespace'}Element> element)
+JSNode* createJS$parameters{'namespace'}Wrapper(ExecState* exec, JSDOMGlobalObject* globalObject, PassRefPtr<$parameters{'namespace'}Element> element)
 {   
-    static HashMap<WebCore::AtomicStringImpl*, Create$parameters{'namespace'}ElementWrapperFunction> map;
+    typedef HashMap<WebCore::AtomicStringImpl*, Create$parameters{'namespace'}ElementWrapperFunction> FunctionMap;
+    DEFINE_STATIC_LOCAL(FunctionMap, map, ());
     if (map.isEmpty()) {
 END
 ;
 
     for my $tag (sort keys %tags) {
-        next if (hasCustomMapping($tag));
+        # Do not add the name to the map if it does not have a JS wrapper constructor or uses the default wrapper.
+        next if usesDefaultJSWrapper($tag, \%tags);
 
-        my $ucTag = $tags{$tag}{"upperCase"};
+        my $conditional = $tags{$tag}{"conditional"};
+        if ($conditional) {
+            my $conditionalString = "ENABLE(" . join(") && ENABLE(", split(/&/, $conditional)) . ")";
+            print F "#if ${conditionalString}\n";
+        }
+
+        my $ucTag = $tags{$tag}{"JSInterfaceName"};
         print F "       map.set(${tag}Tag.localName().impl(), create${ucTag}Wrapper);\n";
-    }
 
-    if ($parameters{'namespace'} eq "HTML") {
-        for my $tag (sort keys %htmlCustomMappings) {
-            next if !$htmlCustomMappings{$tag};
-
-            my $ucCustomTag = $tags{$htmlCustomMappings{$tag}}{"upperCase"};
-            print F "       map.set(${tag}Tag.localName().impl(), create${ucCustomTag}Wrapper);\n";
+        if ($conditional) {
+            print F "#endif\n";
         }
     }
 
-    # Currently SVG has no need to add custom map.set as it only has empty elements
-
     print F <<END
     }
     Create$parameters{'namespace'}ElementWrapperFunction createWrapperFunction = map.get(element->localName().impl());
     if (createWrapperFunction)
-        return createWrapperFunction(exec, element);
-    return new JS$parameters{'namespace'}Element(JS$parameters{'namespace'}ElementPrototype::self(exec), element.get());
+        return createWrapperFunction(exec, globalObject, element);
+    return CREATE_DOM_NODE_WRAPPER(exec, globalObject, $parameters{'namespace'}Element, element.get());
 }
 
 }
@@ -842,16 +1000,17 @@ sub printWrapperFactoryHeaderFile
     print F <<END
 #include <wtf/Forward.h>
 
-namespace KJS {
+namespace JSC {
     class ExecState;
 }                                            
                                              
 namespace WebCore {
 
     class JSNode;
+    class JSDOMGlobalObject;
     class $parameters{'namespace'}Element;
 
-    JSNode* createJS$parameters{'namespace'}Wrapper(KJS::ExecState*, PassRefPtr<$parameters{'namespace'}Element>);
+    JSNode* createJS$parameters{'namespace'}Wrapper(JSC::ExecState*, JSDOMGlobalObject*, PassRefPtr<$parameters{'namespace'}Element>);
 
 }
  
