@@ -1,6 +1,8 @@
 /*
  * Copyright (C) 2006 Apple Computer, Inc.  All rights reserved.
  * Copyright (C) 2007 Alp Toker <alp@atoker.com>
+ * Copyright (C) 2008 Dirk Schulze <vbs85@gmx.de>
+ * Copyright (C) 2008 Nuanti Ltd.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,7 +31,7 @@
 
 #if PLATFORM(CAIRO)
 
-#include "AffineTransform.h"
+#include "TransformationMatrix.h"
 #include "CairoPath.h"
 #include "FloatRect.h"
 #include "Font.h"
@@ -37,7 +39,9 @@
 #include "IntRect.h"
 #include "NotImplemented.h"
 #include "Path.h"
+#include "Pattern.h"
 #include "SimpleFontData.h"
+
 #include <cairo.h>
 #include <math.h>
 #include <stdio.h>
@@ -49,6 +53,7 @@
 #elif PLATFORM(WIN)
 #include <cairo-win32.h>
 #endif
+#include "GraphicsContextPrivate.h"
 #include "GraphicsContextPlatformPrivateCairo.h"
 
 #ifndef M_PI
@@ -78,6 +83,7 @@ GraphicsContext::GraphicsContext(PlatformGraphicsContext* cr)
     , m_data(new GraphicsContextPlatformPrivate)
 {
     m_data->cr = cairo_reference(cr);
+    m_data->syncContext(cr);
     setPaintingDisabled(!cr);
 }
 
@@ -87,12 +93,12 @@ GraphicsContext::~GraphicsContext()
     delete m_data;
 }
 
-AffineTransform GraphicsContext::getCTM() const
+TransformationMatrix GraphicsContext::getCTM() const
 {
     cairo_t* cr = platformContext();
     cairo_matrix_t m;
     cairo_get_matrix(cr, &m);
-    return m;
+    return TransformationMatrix(m.xx, m.yx, m.xy, m.yy, m.x0, m.y0);
 }
 
 cairo_t* GraphicsContext::platformContext() const
@@ -290,32 +296,88 @@ void GraphicsContext::drawEllipse(const IntRect& rect)
     cairo_new_path(cr);
 }
 
-// FIXME: This function needs to be adjusted to match the functionality on the Mac side.
 void GraphicsContext::strokeArc(const IntRect& rect, int startAngle, int angleSpan)
 {
-    if (paintingDisabled())
-        return;
-
-    if (strokeStyle() == NoStroke)
+    if (paintingDisabled() || strokeStyle() == NoStroke)
         return;
 
     int x = rect.x();
     int y = rect.y();
     float w = rect.width();
-#if 0 // FIXME: unused so far
     float h = rect.height();
     float scaleFactor = h / w;
     float reverseScaleFactor = w / h;
-#endif
-    float r = w / 2;
+
+    float hRadius = w / 2;
+    float vRadius = h / 2;
     float fa = startAngle;
     float falen =  fa + angleSpan;
 
     cairo_t* cr = m_data->cr;
     cairo_save(cr);
-    cairo_arc_negative(cr, x + r, y + r, r, -fa * M_PI/180, -falen * M_PI/180);
+
+    if (w != h)
+        cairo_scale(cr, 1., scaleFactor);
+    
+    cairo_arc_negative(cr, x + hRadius, (y + vRadius) * reverseScaleFactor, hRadius, -fa * M_PI/180, -falen * M_PI/180);
+
+    if (w != h)
+        cairo_scale(cr, 1., reverseScaleFactor);
+
+    float width = strokeThickness();
+    int patWidth = 0;
+    
+    switch (strokeStyle()) {
+        case DottedStroke:
+            patWidth = static_cast<int>(width / 2);
+            break;
+        case DashedStroke:
+            patWidth = 3 * static_cast<int>(width / 2);
+            break;
+        default:
+            break;
+    }
+
     setColor(cr, strokeColor());
-    cairo_set_line_width(cr, strokeThickness());
+
+    if (patWidth) {
+        // Example: 80 pixels with a width of 30 pixels.
+        // Remainder is 20.  The maximum pixels of line we could paint
+        // will be 50 pixels.
+        int distance;
+        if (hRadius == vRadius)
+            distance = static_cast<int>((M_PI * hRadius) / 2.0);
+        else // We are elliptical and will have to estimate the distance
+            distance = static_cast<int>((M_PI * sqrtf((hRadius * hRadius + vRadius * vRadius) / 2.0)) / 2.0);
+        
+        int remainder = distance % patWidth;
+        int coverage = distance - remainder;
+        int numSegments = coverage / patWidth;
+
+        float patternOffset = 0.0;
+        // Special case 1px dotted borders for speed.
+        if (patWidth == 1)
+            patternOffset = 1.0;
+        else {
+            bool evenNumberOfSegments = numSegments % 2 == 0;
+            if (remainder)
+                evenNumberOfSegments = !evenNumberOfSegments;
+            if (evenNumberOfSegments) {
+                if (remainder) {
+                    patternOffset += patWidth - remainder;
+                    patternOffset += remainder / 2.0;
+                } else
+                    patternOffset = patWidth / 2.0;
+            } else {
+                if (remainder)
+                    patternOffset = (patWidth - remainder) / 2.0;
+            }
+        }
+
+        double dash = patWidth;
+        cairo_set_dash(cr, &dash, 1, patternOffset);
+    }
+
     cairo_stroke(cr);
     cairo_restore(cr);
 }
@@ -353,13 +415,93 @@ void GraphicsContext::drawConvexPolygon(size_t npoints, const FloatPoint* points
     cairo_restore(cr);
 }
 
-void GraphicsContext::fillRect(const IntRect& rect, const Color& color)
+void GraphicsContext::fillPath()
 {
     if (paintingDisabled())
         return;
 
-    if (color.alpha())
-        fillRectSourceOver(m_data->cr, rect, color);
+    cairo_t* cr = m_data->cr;
+    cairo_save(cr);
+
+    cairo_set_fill_rule(cr, fillRule() == RULE_EVENODD ? CAIRO_FILL_RULE_EVEN_ODD : CAIRO_FILL_RULE_WINDING);
+    switch (m_common->state.fillColorSpace) {
+    case SolidColorSpace:
+        setColor(cr, fillColor());
+        cairo_clip(cr);
+        cairo_paint_with_alpha(cr, m_common->state.globalAlpha);
+        break;
+    case PatternColorSpace: {
+        TransformationMatrix affine;
+        cairo_set_source(cr, m_common->state.fillPattern->createPlatformPattern(affine));
+        cairo_clip(cr);
+        cairo_paint_with_alpha(cr, m_common->state.globalAlpha);
+        break;
+    }
+    case GradientColorSpace:
+        cairo_pattern_t* pattern = m_common->state.fillGradient->platformGradient();
+        cairo_set_source(cr, pattern);
+        cairo_clip(cr);
+        cairo_paint_with_alpha(cr, m_common->state.globalAlpha);
+        break;
+    }
+    cairo_restore(cr);
+}
+
+void GraphicsContext::strokePath()
+{
+    if (paintingDisabled())
+        return;
+
+    cairo_t* cr = m_data->cr;
+    cairo_save(cr);
+    switch (m_common->state.strokeColorSpace) {
+    case SolidColorSpace:
+        float red, green, blue, alpha;
+        strokeColor().getRGBA(red, green, blue, alpha);
+        if (m_common->state.globalAlpha < 1.0f)
+            alpha *= m_common->state.globalAlpha;
+        cairo_set_source_rgba(cr, red, green, blue, alpha);
+        cairo_stroke(cr);
+        break;
+    case PatternColorSpace: {
+        TransformationMatrix affine;
+        cairo_set_source(cr, m_common->state.strokePattern->createPlatformPattern(affine));
+        if (m_common->state.globalAlpha < 1.0f) {
+            cairo_push_group(cr);
+            cairo_paint_with_alpha(cr, m_common->state.globalAlpha);
+            cairo_pop_group_to_source(cr);
+        }
+        cairo_stroke(cr);
+        break;
+    }
+    case GradientColorSpace:
+        cairo_pattern_t* pattern = m_common->state.strokeGradient->platformGradient();
+        cairo_set_source(cr, pattern);
+        if (m_common->state.globalAlpha < 1.0f) {
+            cairo_push_group(cr);
+            cairo_paint_with_alpha(cr, m_common->state.globalAlpha);
+            cairo_pop_group_to_source(cr);
+        }
+        cairo_stroke(cr);
+        break;
+    }
+    cairo_restore(cr);
+}
+
+void GraphicsContext::drawPath()
+{
+    fillPath();
+    strokePath();
+}
+
+void GraphicsContext::fillRect(const FloatRect& rect)
+{
+    if (paintingDisabled())
+        return;
+
+    cairo_t* cr = m_data->cr;
+    cairo_rectangle(cr, rect.x(), rect.y(), rect.width(), rect.height());
+    fillPath();
 }
 
 void GraphicsContext::fillRect(const FloatRect& rect, const Color& color)
@@ -385,32 +527,63 @@ void GraphicsContext::clip(const FloatRect& rect)
     m_data->clip(rect);
 }
 
+void GraphicsContext::clipPath(WindRule clipRule)
+{
+    if (paintingDisabled())
+        return;
+
+    cairo_t* cr = m_data->cr;
+    cairo_set_fill_rule(cr, clipRule == RULE_EVENODD ? CAIRO_FILL_RULE_EVEN_ODD : CAIRO_FILL_RULE_WINDING);
+    cairo_clip(cr);
+}
+
 void GraphicsContext::drawFocusRing(const Color& color)
 {
     if (paintingDisabled())
         return;
 
-    int radius = (focusRingWidth() - 1) / 2;
-    int offset = radius + focusRingOffset();
-
     const Vector<IntRect>& rects = focusRingRects();
     unsigned rectCount = rects.size();
-    IntRect finalFocusRect;
-    for (unsigned i = 0; i < rectCount; i++) {
-        IntRect focusRect = rects[i];
-        focusRect.inflate(offset);
-        finalFocusRect.unite(focusRect);
-    }
 
     cairo_t* cr = m_data->cr;
     cairo_save(cr);
-    // FIXME: These rects should be rounded
-    cairo_rectangle(cr, finalFocusRect.x(), finalFocusRect.y(), finalFocusRect.width(), finalFocusRect.height());
+    cairo_push_group(cr);
+    cairo_new_path(cr);
+
+#if PLATFORM(GTK)
+    GdkRegion* reg = gdk_region_new();
+    for (unsigned i = 0; i < rectCount; i++) {
+        GdkRectangle rect = rects[i];
+        gdk_region_union_with_rect(reg, &rect);
+    }
+    gdk_cairo_region(cr, reg);
+    gdk_region_destroy(reg);
+
+    setColor(cr, color);
+    cairo_set_line_width(cr, 2.0f);
+    setPlatformStrokeStyle(DottedStroke);
+#else
+    int radius = (focusRingWidth() - 1) / 2;
+    for (unsigned i = 0; i < rectCount; i++)
+        addPath(Path::createRoundedRectangle(rects[i], FloatSize(radius, radius)));
 
     // Force the alpha to 50%.  This matches what the Mac does with outline rings.
     Color ringColor(color.red(), color.green(), color.blue(), 127);
     setColor(cr, ringColor);
-    cairo_stroke(cr);
+    cairo_set_line_width(cr, focusRingWidth());
+    setPlatformStrokeStyle(SolidStroke);
+#endif
+
+    cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
+    cairo_stroke_preserve(cr);
+
+    cairo_set_operator(cr, CAIRO_OPERATOR_CLEAR);
+    cairo_set_fill_rule(cr, CAIRO_FILL_RULE_WINDING);
+    cairo_fill(cr);
+
+    cairo_pop_group_to_source(cr);
+    cairo_set_operator(cr, CAIRO_OPERATOR_OVER);
+    cairo_paint(cr);
     cairo_restore(cr);
 }
 
@@ -497,14 +670,14 @@ IntPoint GraphicsContext::origin()
 
 void GraphicsContext::setPlatformFillColor(const Color& col)
 {
-    // FIXME: this is probably a no-op but I'm not sure
-    // notImplemented(); // commented-out because it's chatty and clutters output
+    // Cairo contexts can't hold separate fill and stroke colors
+    // so we set them just before we actually fill or stroke
 }
 
 void GraphicsContext::setPlatformStrokeColor(const Color& col)
 {
-    // FIXME: this is probably a no-op but I'm not sure
-    //notImplemented(); // commented-out because it's chatty and clutters output
+    // Cairo contexts can't hold separate fill and stroke colors
+    // so we set them just before we actually fill or stroke
 }
 
 void GraphicsContext::setPlatformStrokeThickness(float strokeThickness)
@@ -537,9 +710,6 @@ void GraphicsContext::setPlatformStrokeStyle(const StrokeStyle& strokeStyle)
     case DashedStroke:
         cairo_set_dash(m_data->cr, dashPattern, 2, 0);
         break;
-    default:
-        notImplemented();
-        break;
     }
 }
 
@@ -548,14 +718,14 @@ void GraphicsContext::setURLForRect(const KURL& link, const IntRect& destRect)
     notImplemented();
 }
 
-void GraphicsContext::concatCTM(const AffineTransform& transform)
+void GraphicsContext::concatCTM(const TransformationMatrix& transform)
 {
     if (paintingDisabled())
         return;
 
     cairo_t* cr = m_data->cr;
-    const cairo_matrix_t* matrix = reinterpret_cast<const cairo_matrix_t*>(&transform);
-    cairo_transform(cr, matrix);
+    const cairo_matrix_t matrix = cairo_matrix_t(transform);
+    cairo_transform(cr, &matrix);
     m_data->concatCTM(transform);
 }
 
@@ -646,9 +816,8 @@ void GraphicsContext::strokeRect(const FloatRect& rect, float width)
     cairo_t* cr = m_data->cr;
     cairo_save(cr);
     cairo_rectangle(cr, rect.x(), rect.y(), rect.width(), rect.height());
-    setColor(cr, strokeColor());
     cairo_set_line_width(cr, width);
-    cairo_stroke(cr);
+    strokePath();
     cairo_restore(cr);
 }
 
@@ -672,6 +841,11 @@ void GraphicsContext::setLineCap(LineCap lineCap)
     cairo_set_line_cap(m_data->cr, cairoCap);
 }
 
+void GraphicsContext::setLineDash(const DashArray& dashes, float dashOffset)
+{
+    cairo_set_dash(m_data->cr, dashes.data(), dashes.size(), dashOffset);
+}
+
 void GraphicsContext::setLineJoin(LineJoin lineJoin)
 {
     if (paintingDisabled())
@@ -700,9 +874,14 @@ void GraphicsContext::setMiterLimit(float miter)
     cairo_set_miter_limit(m_data->cr, miter);
 }
 
-void GraphicsContext::setAlpha(float)
+void GraphicsContext::setAlpha(float alpha)
 {
-    notImplemented();
+    m_common->state.globalAlpha = alpha;
+}
+
+float GraphicsContext::getAlpha()
+{
+    return m_common->state.globalAlpha;
 }
 
 static inline cairo_operator_t toCairoOperator(CompositeOperator op)
@@ -791,6 +970,7 @@ void GraphicsContext::clipOut(const Path& path)
     if (paintingDisabled())
         return;
 
+#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1,4,0)
     cairo_t* cr = m_data->cr;
     double x1, y1, x2, y2;
     cairo_clip_extents(cr, &x1, &y1, &x2, &y2);
@@ -801,6 +981,9 @@ void GraphicsContext::clipOut(const Path& path)
     cairo_set_fill_rule(cr, CAIRO_FILL_RULE_EVEN_ODD);
     cairo_clip(cr);
     cairo_set_fill_rule(cr, savedFillRule);
+#else
+    notImplemented();
+#endif
 }
 
 void GraphicsContext::rotate(float radians)
@@ -826,6 +1009,7 @@ void GraphicsContext::clipOut(const IntRect& r)
     if (paintingDisabled())
         return;
 
+#if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1,4,0)
     cairo_t* cr = m_data->cr;
     double x1, y1, x2, y2;
     cairo_clip_extents(cr, &x1, &y1, &x2, &y2);
@@ -835,6 +1019,9 @@ void GraphicsContext::clipOut(const IntRect& r)
     cairo_set_fill_rule(cr, CAIRO_FILL_RULE_EVEN_ODD);
     cairo_clip(cr);
     cairo_set_fill_rule(cr, savedFillRule);
+#else
+    notImplemented();
+#endif
 }
 
 void GraphicsContext::clipOutEllipseInRect(const IntRect& r)
@@ -879,20 +1066,9 @@ GdkDrawable* GraphicsContext::gdkDrawable() const
 
     return GDK_DRAWABLE(m_data->expose->window);
 }
-
-IntPoint GraphicsContext::translatePoint(const IntPoint& point) const
-{
-    cairo_matrix_t tm;
-    cairo_get_matrix(m_data->cr, &tm);
-    double x = point.x();
-    double y = point.y();
-
-    cairo_matrix_transform_point(&tm, &x, &y);
-    return IntPoint(x, y);
-}
 #endif
 
-void GraphicsContext::setUseAntialiasing(bool enable)
+void GraphicsContext::setPlatformShouldAntialias(bool enable)
 {
     if (paintingDisabled())
         return;
@@ -903,6 +1079,15 @@ void GraphicsContext::setUseAntialiasing(bool enable)
     cairo_set_antialias(m_data->cr, enable ? CAIRO_ANTIALIAS_DEFAULT : CAIRO_ANTIALIAS_NONE);
 }
 
+void GraphicsContext::setImageInterpolationQuality(InterpolationQuality)
+{
+}
+
+InterpolationQuality GraphicsContext::imageInterpolationQuality() const
+{
+    return InterpolationDefault;
+}
+
 } // namespace WebCore
 
 #endif // PLATFORM(CAIRO)
