@@ -30,7 +30,6 @@
 #include "Document.h"
 #include "DocumentLoader.h"
 #include "Element.h"
-#include "EventNames.h"
 #include "FrameLoader.h"
 #include "FrameTree.h"
 #include "Frame.h"
@@ -47,29 +46,33 @@
 #include "Page.h"
 #include "FocusController.h"
 #include "PlatformMouseEvent.h"
-#if PLATFORM(WIN)
+#if PLATFORM(WIN_OS) && !PLATFORM(WX) && ENABLE(NETSCAPE_PLUGIN_API)
 #include "PluginMessageThrottlerWin.h"
 #endif
 #include "PluginPackage.h"
 #include "JSDOMBinding.h"
 #include "ScriptController.h"
+#include "ScriptValue.h"
 #include "PluginDatabase.h"
 #include "PluginDebug.h"
+#include "PluginMainThreadScheduler.h"
 #include "PluginPackage.h"
+#include "RenderBox.h"
+#include "RenderObject.h"
 #include "c_instance.h"
 #include "npruntime_impl.h"
 #include "runtime_root.h"
 #include "Settings.h"
 #include "runtime.h"
-#include <kjs/JSLock.h>
-#include <kjs/value.h>
+#include <runtime/JSLock.h>
+#include <runtime/JSValue.h>
 #include <wtf/ASCIICType.h>
 
-using KJS::ExecState;
-using KJS::JSLock;
-using KJS::JSObject;
-using KJS::JSValue;
-using KJS::UString;
+using JSC::ExecState;
+using JSC::JSLock;
+using JSC::JSObject;
+using JSC::JSValue;
+using JSC::UString;
 
 using std::min;
 
@@ -77,14 +80,13 @@ using namespace WTF;
 
 namespace WebCore {
 
-using namespace EventNames;
 using namespace HTMLNames;
 
 static int s_callingPlugin;
 
 static String scriptStringIfJavaScriptURL(const KURL& url)
 {
-    if (!url.protocolIs("javascript"))
+    if (!protocolIsJavaScript(url))
         return String();
 
     // This returns an unescaped string
@@ -111,24 +113,29 @@ IntRect PluginView::windowClipRect() const
     return clipRect;
 }
 
-void PluginView::setFrameGeometry(const IntRect& rect)
+void PluginView::setFrameRect(const IntRect& rect)
 {
     if (m_element->document()->printing())
         return;
 
-    if (rect != frameGeometry())
-        Widget::setFrameGeometry(rect);
+    if (rect != frameRect())
+        Widget::setFrameRect(rect);
 
-    updateWindow();
+    updatePluginWidget();
 
-#if PLATFORM(WIN)
+#if PLATFORM(WIN_OS)
+    // On Windows, always call plugin to change geometry.
     setNPWindowRect(rect);
+#elif XP_UNIX
+    // On Unix, only call plugin if it's full-page.
+    if (m_mode == NP_FULL)
+        setNPWindowRect(rect);
 #endif
 }
 
-void PluginView::geometryChanged() const
+void PluginView::frameRectsChanged()
 {
-    updateWindow();
+    updatePluginWidget();
 }
 
 void PluginView::handleEvent(Event* event)
@@ -147,13 +154,15 @@ bool PluginView::start()
     if (m_isStarted)
         return false;
 
+    PluginMainThreadScheduler::scheduler().registerPlugin(m_instance);
+
     ASSERT(m_plugin);
     ASSERT(m_plugin->pluginFuncs()->newp);
 
     NPError npErr;
     {
         PluginView::setCurrentPluginView(this);
-        KJS::JSLock::DropAllLocks dropAllLocks;
+        JSC::JSLock::DropAllLocks dropAllLocks(JSC::SilenceAssertionsOnly);
         setCallingPlugin(true);
         npErr = m_plugin->pluginFuncs()->newp((NPMIMEType)m_mimeType.data(), m_instance, m_mode, m_paramCount, m_paramNames, m_paramValues, NULL);
         setCallingPlugin(false);
@@ -196,14 +205,14 @@ static char* createUTF8String(const String& str)
     return result;
 }
 
-static bool getString(ScriptController* proxy, JSValue* result, String& string)
+static bool getString(ScriptController* proxy, JSValue result, String& string)
 {
-    if (!proxy || !result || result->isUndefined())
+    if (!proxy || !result || result.isUndefined())
         return false;
-    JSLock lock;
+    JSLock lock(JSC::SilenceAssertionsOnly);
 
     ExecState* exec = proxy->globalObject()->globalExec();
-    UString ustring = result->toString(exec);
+    UString ustring = result.toString(exec);
     exec->clearException();
 
     string = ustring;
@@ -230,12 +239,16 @@ void PluginView::performRequest(PluginRequest* request)
             m_streams.add(stream);
             stream->start();
         } else {
-            m_parentFrame->loader()->load(request->frameLoadRequest().resourceRequest(), targetFrameName);
-      
+            // If the target frame is our frame, we could destroy the
+            // PluginView, so we protect it. <rdar://problem/6991251>
+            RefPtr<PluginView> protect(this);
+
+            m_parentFrame->loader()->load(request->frameLoadRequest().resourceRequest(), targetFrameName, false);
+
             // FIXME: <rdar://problem/4807469> This should be sent when the document has finished loading
             if (request->sendNotification()) {
                 PluginView::setCurrentPluginView(this);
-                KJS::JSLock::DropAllLocks dropAllLocks;
+                JSC::JSLock::DropAllLocks dropAllLocks(JSC::SilenceAssertionsOnly);
                 setCallingPlugin(true);
                 m_plugin->pluginFuncs()->urlnotify(m_instance, requestURL.string().utf8().data(), NPRES_DONE, request->notifyData());
                 setCallingPlugin(false);
@@ -251,13 +264,13 @@ void PluginView::performRequest(PluginRequest* request)
     
     // Executing a script can cause the plugin view to be destroyed, so we keep a reference to the parent frame.
     RefPtr<Frame> parentFrame = m_parentFrame;
-    JSValue* result = m_parentFrame->loader()->executeScript(jsString, request->shouldAllowPopups());
+    JSValue result = m_parentFrame->loader()->executeScript(jsString, request->shouldAllowPopups()).jsValue();
 
     if (targetFrameName.isNull()) {
         String resultString;
 
         CString cstr;
-        if (getString(parentFrame->scriptProxy(), result, resultString))
+        if (getString(parentFrame->script(), result, resultString))
             cstr = resultString.utf8();
 
         RefPtr<PluginStream> stream = PluginStream::create(this, m_parentFrame, request->frameLoadRequest().resourceRequest(), request->sendNotification(), request->notifyData(), plugin()->pluginFuncs(), instance(), m_plugin->quirks());
@@ -301,6 +314,10 @@ NPError PluginView::load(const FrameLoadRequest& frameLoadRequest, bool sendNoti
     if (url.isEmpty())
         return NPERR_INVALID_URL;
 
+    // Don't allow requests to be made when the document loader is stopping all loaders.
+    if (m_parentFrame->loader()->documentLoader()->isStopping())
+        return NPERR_GENERIC_ERROR;
+
     const String& targetFrameName = frameLoadRequest.frameName();
     String jsString = scriptStringIfJavaScriptURL(url);
 
@@ -314,6 +331,8 @@ NPError PluginView::load(const FrameLoadRequest& frameLoadRequest, bool sendNoti
         // For security reasons, only allow JS requests to be made on the frame that contains the plug-in.
         if (!targetFrameName.isNull() && m_parentFrame->tree()->find(targetFrameName) != m_parentFrame)
             return NPERR_INVALID_PARAM;
+    } else if (!FrameLoader::canLoad(url, String(), m_parentFrame->document())) {
+            return NPERR_GENERIC_ERROR;
     }
 
     PluginRequest* request = new PluginRequest(frameLoadRequest, sendNotification, notifyData, arePopupsAllowed());
@@ -382,12 +401,12 @@ int32 PluginView::write(NPStream* stream, int32 len, void* buffer)
 
 NPError PluginView::destroyStream(NPStream* stream, NPReason reason)
 {
-    PluginStream* browserStream = static_cast<PluginStream*>(stream->ndata);
-
     if (!stream || PluginStream::ownerForStream(stream) != m_instance)
         return NPERR_INVALID_INSTANCE_ERROR;
 
+    PluginStream* browserStream = static_cast<PluginStream*>(stream->ndata);
     browserStream->cancelAndDestroyStream(reason);
+
     return NPERR_NO_ERROR;
 }
 
@@ -399,6 +418,8 @@ void PluginView::status(const char* message)
 
 NPError PluginView::setValue(NPPVariable variable, void* value)
 {
+    LOG(Plugins, "PluginView::setValue(%s): ", prettyNameForNPPVariable(variable, value).data());
+
     switch (variable) {
     case NPPVpluginWindowBool:
         m_isWindowed = value;
@@ -406,6 +427,50 @@ NPError PluginView::setValue(NPPVariable variable, void* value)
     case NPPVpluginTransparentBool:
         m_isTransparent = value;
         return NPERR_NO_ERROR;
+#if defined(XP_MACOSX)
+    case NPPVpluginDrawingModel: {
+        // Can only set drawing model inside NPP_New()
+        if (this != currentPluginView())
+           return NPERR_GENERIC_ERROR;
+
+        NPDrawingModel newDrawingModel = NPDrawingModel(uintptr_t(value));
+        switch (newDrawingModel) {
+        case NPDrawingModelCoreGraphics:
+            m_drawingModel = newDrawingModel;
+            return NPERR_NO_ERROR;
+#ifndef NP_NO_QUICKDRAW
+        case NPDrawingModelQuickDraw:
+#endif
+        case NPDrawingModelCoreAnimation:
+        default:
+            LOG(Plugins, "Plugin asked for unsupported drawing model: %s",
+                    prettyNameForDrawingModel(newDrawingModel));
+            return NPERR_GENERIC_ERROR;
+        }
+    }
+
+    case NPPVpluginEventModel: {
+        // Can only set event model inside NPP_New()
+        if (this != currentPluginView())
+           return NPERR_GENERIC_ERROR;
+
+        NPEventModel newEventModel = NPEventModel(uintptr_t(value));
+        switch (newEventModel) {
+#ifndef NP_NO_CARBON
+        case NPEventModelCarbon:
+#endif
+        case NPEventModelCocoa:
+            m_eventModel = newEventModel;
+            return NPERR_NO_ERROR;
+
+        default:
+            LOG(Plugins, "Plugin asked for unsupported event model: %s",
+                    prettyNameForEventModel(newEventModel));
+            return NPERR_GENERIC_ERROR;
+        }
+    }
+#endif // defined(XP_MACOSX)
+
     default:
         notImplemented();
         return NPERR_GENERIC_ERROR;
@@ -417,7 +482,7 @@ void PluginView::invalidateTimerFired(Timer<PluginView>* timer)
     ASSERT(timer == &m_invalidateTimer);
 
     for (unsigned i = 0; i < m_invalidRects.size(); i++)
-        Widget::invalidateRect(m_invalidRects[i]);
+        invalidateRect(m_invalidRects[i]);
     m_invalidRects.clear();
 }
 
@@ -452,7 +517,7 @@ void PluginView::setJavaScriptPaused(bool paused)
         m_requestTimer.startOneShot(0);
 }
 
-PassRefPtr<KJS::Bindings::Instance> PluginView::bindingInstance()
+PassRefPtr<JSC::Bindings::Instance> PluginView::bindingInstance()
 {
 #if ENABLE(NETSCAPE_PLUGIN_API)
     NPObject* object = 0;
@@ -460,21 +525,33 @@ PassRefPtr<KJS::Bindings::Instance> PluginView::bindingInstance()
     if (!m_plugin || !m_plugin->pluginFuncs()->getvalue)
         return 0;
 
+    // On Windows, calling Java's NPN_GetValue can allow the message loop to
+    // run, allowing loading to take place or JavaScript to run. Protect the
+    // PluginView from destruction. <rdar://problem/6978804>
+    RefPtr<PluginView> protect(this);
+
     NPError npErr;
     {
         PluginView::setCurrentPluginView(this);
-        KJS::JSLock::DropAllLocks dropAllLocks;
+        JSC::JSLock::DropAllLocks dropAllLocks(JSC::SilenceAssertionsOnly);
         setCallingPlugin(true);
         npErr = m_plugin->pluginFuncs()->getvalue(m_instance, NPPVpluginScriptableNPObject, &object);
         setCallingPlugin(false);
         PluginView::setCurrentPluginView(0);
     }
 
+    if (hasOneRef()) {
+        // The renderer for the PluginView was destroyed during the above call, and
+        // the PluginView will be destroyed when this function returns, so we
+        // return null.
+        return 0;
+    }
+
     if (npErr != NPERR_NO_ERROR || !object)
         return 0;
 
-    RefPtr<KJS::Bindings::RootObject> root = m_parentFrame->createRootObject(this, m_parentFrame->scriptProxy()->globalObject());
-    RefPtr<KJS::Bindings::Instance> instance = KJS::Bindings::CInstance::create(object, root.release());
+    RefPtr<JSC::Bindings::RootObject> root = m_parentFrame->script()->createRootObject(this);
+    RefPtr<JSC::Bindings::Instance> instance = JSC::Bindings::CInstance::create(object, root.release());
 
     _NPN_ReleaseObject(object);
 
@@ -527,20 +604,29 @@ PluginView::PluginView(Frame* parentFrame, const IntSize& size, PluginPackage* p
     , m_popPopupsStateTimer(this, &PluginView::popPopupsStateTimerFired)
     , m_paramNames(0)
     , m_paramValues(0)
+#if defined(XP_MACOSX)
+    , m_isWindowed(false)
+#else
     , m_isWindowed(true)
+#endif
     , m_isTransparent(false)
-    , m_isVisible(false)
-    , m_attachedToWindow(false)
     , m_haveInitialized(false)
 #if PLATFORM(GTK) || defined(Q_WS_X11)
     , m_needsXEmbed(false)
 #endif
-#if PLATFORM(WIN)
+#if PLATFORM(WIN_OS) && !PLATFORM(WX) && ENABLE(NETSCAPE_PLUGIN_API)
     , m_pluginWndProc(0)
     , m_lastMessage(0)
     , m_isCallingPluginWndProc(false)
+    , m_wmPrintHDC(0)
 #endif
+#if (PLATFORM(QT) && PLATFORM(WIN_OS)) || defined(XP_MACOSX)
     , m_window(0)
+#endif
+#if defined(XP_MACOSX)
+    , m_drawingModel(NPDrawingModel(-1))
+    , m_eventModel(NPEventModel(-1))
+#endif
     , m_loadManually(loadManually)
     , m_manualStream(0)
     , m_isJavaScriptPaused(false)
@@ -558,8 +644,9 @@ PluginView::PluginView(Frame* parentFrame, const IntSize& size, PluginPackage* p
 
     setParameters(paramNames, paramValues);
 
-#ifdef XP_UNIX
-    m_npWindow.ws_info = 0;
+    memset(&m_npWindow, 0, sizeof(m_npWindow));
+#if defined(XP_MACOSX)
+    memset(&m_npCgContext, 0, sizeof(m_npCgContext));
 #endif
 
     m_mode = m_loadManually ? NP_FULL : NP_EMBED;
@@ -567,8 +654,19 @@ PluginView::PluginView(Frame* parentFrame, const IntSize& size, PluginPackage* p
     resize(size);
 }
 
+void PluginView::focusPluginElement()
+{
+    // Focus the plugin
+    if (Page* page = m_parentFrame->page())
+        page->focusController()->setFocusedFrame(m_parentFrame);
+    m_parentFrame->document()->setFocusedNode(m_element);
+}
+
 void PluginView::didReceiveResponse(const ResourceResponse& response)
 {
+    if (m_status != PluginStatusLoadedSuccessfully)
+        return;
+
     ASSERT(m_loadManually);
     ASSERT(!m_manualStream);
 
@@ -580,6 +678,9 @@ void PluginView::didReceiveResponse(const ResourceResponse& response)
 
 void PluginView::didReceiveData(const char* data, int length)
 {
+    if (m_status != PluginStatusLoadedSuccessfully)
+        return;
+
     ASSERT(m_loadManually);
     ASSERT(m_manualStream);
     
@@ -588,6 +689,9 @@ void PluginView::didReceiveData(const char* data, int length)
 
 void PluginView::didFinishLoading()
 {
+    if (m_status != PluginStatusLoadedSuccessfully)
+        return;
+
     ASSERT(m_loadManually);
     ASSERT(m_manualStream);
 
@@ -596,6 +700,9 @@ void PluginView::didFinishLoading()
 
 void PluginView::didFail(const ResourceError& error)
 {
+    if (m_status != PluginStatusLoadedSuccessfully)
+        return;
+
     ASSERT(m_loadManually);
     ASSERT(m_manualStream);
 
@@ -620,7 +727,7 @@ bool PluginView::isCallingPlugin()
     return s_callingPlugin > 0;
 }
 
-PluginView* PluginView::create(Frame* parentFrame, const IntSize& size, Element* element, const KURL& url, const Vector<String>& paramNames, const Vector<String>& paramValues, const String& mimeType, bool loadManually)
+PassRefPtr<PluginView> PluginView::create(Frame* parentFrame, const IntSize& size, Element* element, const KURL& url, const Vector<String>& paramNames, const Vector<String>& paramValues, const String& mimeType, bool loadManually)
 {
     // if we fail to find a plugin for this MIME type, findPlugin will search for
     // a plugin by the file extension and update the MIME type, so pass a mutable String
@@ -633,7 +740,7 @@ PluginView* PluginView::create(Frame* parentFrame, const IntSize& size, Element*
         plugin = PluginDatabase::installedPlugins()->findPlugin(url, mimeTypeCopy);
     }
 
-    return new PluginView(parentFrame, size, plugin, element, url, paramNames, paramValues, mimeTypeCopy, loadManually);
+    return adoptRef(new PluginView(parentFrame, size, plugin, element, url, paramNames, paramValues, mimeTypeCopy, loadManually));
 }
 
 void PluginView::freeStringArray(char** stringArray, int length)
@@ -861,4 +968,40 @@ NPError PluginView::handlePost(const char* url, const char* target, uint32 len,
     return load(frameLoadRequest, sendNotification, notifyData);
 }
 
+void PluginView::invalidateWindowlessPluginRect(const IntRect& rect)
+{
+    if (!isVisible())
+        return;
+    
+    if (!m_element->renderer())
+        return;
+    RenderBox* renderer = toRenderBox(m_element->renderer());
+    
+    IntRect dirtyRect = rect;
+    dirtyRect.move(renderer->borderLeft() + renderer->paddingLeft(), renderer->borderTop() + renderer->paddingTop());
+    renderer->repaintRectangle(dirtyRect);
+}
+
+void PluginView::paintMissingPluginIcon(GraphicsContext* context, const IntRect& rect)
+{
+    static RefPtr<Image> nullPluginImage;
+    if (!nullPluginImage)
+        nullPluginImage = Image::loadPlatformResource("nullPlugin");
+
+    IntRect imageRect(frameRect().x(), frameRect().y(), nullPluginImage->width(), nullPluginImage->height());
+
+    int xOffset = (frameRect().width() - imageRect.width()) / 2;
+    int yOffset = (frameRect().height() - imageRect.height()) / 2;
+
+    imageRect.move(xOffset, yOffset);
+
+    if (!rect.intersects(imageRect))
+        return;
+
+    context->save();
+    context->clip(windowClipRect());
+    context->drawImage(nullPluginImage.get(), imageRect.location());
+    context->restore();
+}
+
 } // namespace WebCore
