@@ -28,6 +28,7 @@
 #include "CharacterNames.h"
 #include "RenderLayer.h"
 #include "RenderView.h"
+#include <wtf/StdLibExtras.h>
 
 using namespace std;
 
@@ -35,7 +36,8 @@ namespace WebCore {
 
 class FlexBoxIterator {
 public:
-    FlexBoxIterator(RenderFlexibleBox* parent) {
+    FlexBoxIterator(RenderFlexibleBox* parent)
+    {
         box = parent;
         if (box->style()->boxOrient() == HORIZONTAL && box->style()->direction() == RTL)
             forward = box->style()->boxDirection() != BNORMAL;
@@ -44,45 +46,47 @@ public:
         lastOrdinal = 1; 
         if (!forward) {
             // No choice, since we're going backwards, we have to find out the highest ordinal up front.
-            RenderObject* child = box->firstChild();
+            RenderBox* child = box->firstChildBox();
             while (child) {
                 if (child->style()->boxOrdinalGroup() > lastOrdinal)
                     lastOrdinal = child->style()->boxOrdinalGroup();
-                child = child->nextSibling();
+                child = child->nextSiblingBox();
             }
         }
         
         reset();
     }
 
-    void reset() {
+    void reset()
+    {
         current = 0;
         currentOrdinal = forward ? 0 : lastOrdinal+1;
     }
 
-    RenderObject* first() {
+    RenderBox* first()
+    {
         reset();
         return next();
     }
     
-    RenderObject* next() {
-
+    RenderBox* next()
+    {
         do { 
             if (!current) {
                 if (forward) {
                     currentOrdinal++; 
                     if (currentOrdinal > lastOrdinal)
                         return 0;
-                    current = box->firstChild();
+                    current = box->firstChildBox();
                 } else {
                     currentOrdinal--;
                     if (currentOrdinal == 0)
                         return 0;
-                    current = box->lastChild();
+                    current = box->lastChildBox();
                 }
             }
             else
-                current = forward ? current->nextSibling() : current->previousSibling();
+                current = forward ? current->nextSiblingBox() : current->previousSiblingBox();
             if (current && current->style()->boxOrdinalGroup() > lastOrdinal)
                 lastOrdinal = current->style()->boxOrdinalGroup();
         } while (!current || current->style()->boxOrdinalGroup() != currentOrdinal ||
@@ -92,7 +96,7 @@ public:
 
 private:
     RenderFlexibleBox* box;
-    RenderObject* current;
+    RenderBox* current;
     bool forward;
     unsigned int currentOrdinal;
     unsigned int lastOrdinal;
@@ -111,13 +115,10 @@ RenderFlexibleBox::~RenderFlexibleBox()
 
 void RenderFlexibleBox::calcHorizontalPrefWidths()
 {
-    RenderObject *child = firstChild();
-    while (child) {
+    for (RenderBox* child = firstChildBox(); child; child = child->nextSiblingBox()) {
         // positioned children don't affect the minmaxwidth
-        if (child->isPositioned() || child->style()->visibility() == COLLAPSE) {
-            child = child->nextSibling();
+        if (child->isPositioned() || child->style()->visibility() == COLLAPSE)
             continue;
-        }
 
         // A margin basically has three types: fixed, percentage, and auto (variable).
         // Auto and percentage margins simply become 0 when computing min/max width.
@@ -133,21 +134,15 @@ void RenderFlexibleBox::calcHorizontalPrefWidths()
 
         m_minPrefWidth += child->minPrefWidth() + margin;
         m_maxPrefWidth += child->maxPrefWidth() + margin;
-
-        child = child->nextSibling();
     }    
 }
 
 void RenderFlexibleBox::calcVerticalPrefWidths()
 {
-    RenderObject *child = firstChild();
-    while(child != 0)
-    {
+    for (RenderBox* child = firstChildBox(); child; child = child->nextSiblingBox()) {
         // Positioned children and collapsed children don't affect the min/max width
-        if (child->isPositioned() || child->style()->visibility() == COLLAPSE) {
-            child = child->nextSibling();
+        if (child->isPositioned() || child->style()->visibility() == COLLAPSE)
             continue;
-        }
 
         // A margin basically has three types: fixed, percentage, and auto (variable).
         // Auto/percentage margins simply become 0 when computing min/max width.
@@ -165,8 +160,6 @@ void RenderFlexibleBox::calcVerticalPrefWidths()
         
         w = child->maxPrefWidth() + margin;
         m_maxPrefWidth = max(w, m_maxPrefWidth);
-
-        child = child->nextSibling();
     }    
 }
 
@@ -198,6 +191,10 @@ void RenderFlexibleBox::calcPrefWidths()
     }
 
     int toAdd = borderLeft() + borderRight() + paddingLeft() + paddingRight();
+    
+    if (hasOverflowClip() && style()->overflowY() == OSCROLL)
+        toAdd += verticalScrollbarWidth();
+
     m_minPrefWidth += toAdd;
     m_maxPrefWidth += toAdd;
 
@@ -211,32 +208,22 @@ void RenderFlexibleBox::layoutBlock(bool relayoutChildren)
     if (!relayoutChildren && layoutOnlyPositionedObjects())
         return;
 
-    IntRect oldBounds;
-    IntRect oldOutlineBox;
-    bool checkForRepaint = checkForRepaintDuringLayout();
-    if (checkForRepaint) {
-        oldBounds = absoluteClippedOverflowRect();
-        oldOutlineBox = absoluteOutlineBox();
-    }
+    LayoutRepainter repainter(*this, checkForRepaintDuringLayout());
+    LayoutStateMaintainer statePusher(view(), this, IntSize(x(), y()), hasTransform() || hasReflection());
 
-    if (!hasReflection())
-        view()->pushLayoutState(this, IntSize(m_x, m_y));
-    else
-        view()->disableLayoutState();
-
-    int previousWidth = m_width;
-    int previousHeight = m_height;
+    int previousWidth = width();
+    int previousHeight = height();
     
     calcWidth();
     calcHeight();
-    m_overflowWidth = m_width;
+    m_overflowWidth = width();
 
-    if (previousWidth != m_width || previousHeight != m_height ||
+    if (previousWidth != width() || previousHeight != height() ||
         (parent()->isFlexibleBox() && parent()->style()->boxOrient() == HORIZONTAL &&
          parent()->style()->boxAlign() == BSTRETCH))
         relayoutChildren = true;
 
-    m_height = 0;
+    setHeight(0);
     m_overflowHeight = 0;
     m_flexingChildren = m_stretchingChildren = false;
 
@@ -245,9 +232,9 @@ void RenderFlexibleBox::layoutBlock(bool relayoutChildren)
     // For overflow:scroll blocks, ensure we have both scrollbars in place always.
     if (scrollsOverflow()) {
         if (style()->overflowX() == OSCROLL)
-            m_layer->setHasHorizontalScrollbar(true);
+            layer()->setHasHorizontalScrollbar(true);
         if (style()->overflowY() == OSCROLL)
-            m_layer->setHasVerticalScrollbar(true);
+            layer()->setHasVerticalScrollbar(true);
     }
 
     if (isHorizontal())
@@ -255,21 +242,21 @@ void RenderFlexibleBox::layoutBlock(bool relayoutChildren)
     else
         layoutVerticalBox(relayoutChildren);
 
-    int oldHeight = m_height;
+    int oldHeight = height();
     calcHeight();
-    if (oldHeight != m_height) {
+    if (oldHeight != height()) {
         // If the block got expanded in size, then increase our overflowheight to match.
-        if (m_overflowHeight > m_height)
+        if (m_overflowHeight > height())
             m_overflowHeight -= (borderBottom() + paddingBottom() + horizontalScrollbarHeight());
-        if (m_overflowHeight < m_height)
-            m_overflowHeight = m_height;
+        if (m_overflowHeight < height())
+            m_overflowHeight = height();
     }
-    if (previousHeight != m_height)
+    if (previousHeight != height())
         relayoutChildren = true;
 
     layoutPositionedObjects(relayoutChildren || isRoot());
 
-    if (!isFloatingOrPositioned() && m_height == 0) {
+    if (!isFloatingOrPositioned() && height() == 0) {
         // We are a block with no border and padding and a computed height
         // of 0.  The CSS spec states that zero-height blocks collapse their margins
         // together.
@@ -288,15 +275,15 @@ void RenderFlexibleBox::layoutBlock(bool relayoutChildren)
     }
 
     // Always ensure our overflow width is at least as large as our width.
-    if (m_overflowWidth < m_width)
-        m_overflowWidth = m_width;
+    if (m_overflowWidth < width())
+        m_overflowWidth = width();
 
     if (!hasOverflowClip()) {
         for (ShadowData* boxShadow = style()->boxShadow(); boxShadow; boxShadow = boxShadow->next) {
-            m_overflowLeft = min(m_overflowLeft, boxShadow->x - boxShadow->blur);
-            m_overflowWidth = max(m_overflowWidth, m_width + boxShadow->x + boxShadow->blur);
-            m_overflowTop = min(m_overflowTop, boxShadow->y - boxShadow->blur);
-            m_overflowHeight = max(m_overflowHeight, m_height + boxShadow->y + boxShadow->blur);
+            m_overflowLeft = min(m_overflowLeft, boxShadow->x - boxShadow->blur - boxShadow->spread);
+            m_overflowWidth = max(m_overflowWidth, width() + boxShadow->x + boxShadow->blur + boxShadow->spread);
+            m_overflowTop = min(m_overflowTop, boxShadow->y - boxShadow->blur - boxShadow->spread);
+            m_overflowHeight = max(m_overflowHeight, height() + boxShadow->y + boxShadow->blur + boxShadow->spread);
         }
         
         if (hasReflection()) {
@@ -308,19 +295,15 @@ void RenderFlexibleBox::layoutBlock(bool relayoutChildren)
         }
     }
 
-    if (!hasReflection())
-        view()->popLayoutState();
-    else
-        view()->enableLayoutState();
+    statePusher.pop();
 
     // Update our scrollbars if we're overflow:auto/scroll/hidden now that we know if
     // we overflow or not.
     if (hasOverflowClip())
-        m_layer->updateScrollInfoAfterLayout();
+        layer()->updateScrollInfoAfterLayout();
 
     // Repaint with our new bounds if they are different from our old bounds.
-    if (checkForRepaint)
-        repaintAfterLayoutIfNeeded(oldBounds, oldOutlineBox);
+    repainter.repaintAfterLayout();
     
     setNeedsLayout(false);
 }
@@ -337,11 +320,11 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
     unsigned int lowestFlexGroup = 0;
     bool haveFlex = false;
     int remainingSpace = 0;
-    m_overflowHeight = m_height;
+    m_overflowHeight = height();
 
     // The first walk over our kids is to find out if we have any flexible children.
     FlexBoxIterator iterator(this);
-    RenderObject* child = iterator.next();
+    RenderBox* child = iterator.next();
     while (child) {
         // Check to see if this child flexes.
         if (!child->isPositioned() && child->style()->boxFlex() > 0.0f) {
@@ -361,13 +344,15 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
         }
         child = iterator.next();
     }
-    
+
+    RenderBlock::startDelayUpdateScrollInfo();
+
     // We do 2 passes.  The first pass is simply to lay everyone out at
     // their preferred widths.  The second pass handles flexing the children.
     do {
         // Reset our height.
-        m_height = yPos;
-        m_overflowHeight = m_height;
+        setHeight(yPos);
+        m_overflowHeight = height();
         xPos = borderLeft() + paddingLeft();
                 
         // Our first pass is done without flexing.  We simply lay the children
@@ -393,9 +378,10 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
     
             // Update our height and overflow height.
             if (style()->boxAlign() == BBASELINE) {
-                int ascent = child->marginTop() + child->getBaselineOfFirstLineBox();
+                int ascent = child->firstLineBoxBaseline();
                 if (ascent == -1)
-                    ascent = child->marginTop() + child->height() + child->marginBottom();
+                    ascent = child->height() + child->marginBottom();
+                ascent += child->marginTop();
                 int descent = (child->marginTop() + child->height() + child->marginBottom()) - ascent;
                 
                 // Update our maximum ascent.
@@ -405,28 +391,28 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
                 maxDescent = max(maxDescent, descent);
                 
                 // Now update our height.
-                m_height = max(yPos + maxAscent + maxDescent, m_height);
+                setHeight(max(yPos + maxAscent + maxDescent, height()));
             }
             else
-                m_height = max(m_height, yPos + child->marginTop() + child->height() + child->marginBottom());
+                setHeight(max(height(), yPos + child->marginTop() + child->height() + child->marginBottom()));
 
             child = iterator.next();
         }
         
         if (!iterator.first() && hasLineIfEmpty())
-            m_height += lineHeight(true, true);
+            setHeight(height() + lineHeight(true, true));
         
-        m_height += toAdd;
+        setHeight(height() + toAdd);
 
         // Always make sure our overflowheight is at least our height.
-        if (m_overflowHeight < m_height)
-            m_overflowHeight = m_height;
+        if (m_overflowHeight < height())
+            m_overflowHeight = height();
         
-        oldHeight = m_height;
+        oldHeight = height();
         calcHeight();
 
         relayoutChildren = false;
-        if (oldHeight != m_height)
+        if (oldHeight != height())
             heightSpecified = true;
         
         // Now that our height is actually known, we can place our boxes.
@@ -435,13 +421,13 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
         while (child) {
             if (child->isPositioned()) {
                 child->containingBlock()->insertPositionedObject(child);
-                if (child->hasStaticX()) {
+                if (child->style()->hasStaticX()) {
                     if (style()->direction() == LTR)
-                        child->setStaticX(xPos);
-                    else child->setStaticX(width() - xPos);
+                        child->layer()->setStaticX(xPos);
+                    else child->layer()->setStaticX(width() - xPos);
                 }
-                if (child->hasStaticY())
-                    child->setStaticY(yPos);
+                if (child->style()->hasStaticY())
+                    child->layer()->setStaticY(yPos);
                 child = iterator.next();
                 continue;
             }
@@ -450,7 +436,7 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
             // fill the height of a containing box by default.
             // Now do a layout.
             int oldChildHeight = child->height();
-            static_cast<RenderBox*>(child)->calcHeight();
+            child->calcHeight();
             if (oldChildHeight != child->height())
                 child->setChildNeedsLayout(true, false);
             child->layoutIfNeeded();
@@ -463,9 +449,10 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
                     childY += child->marginTop() + max(0, (contentHeight() - (child->height() + child->marginTop() + child->marginBottom()))/2);
                     break;
                 case BBASELINE: {
-                    int ascent = child->marginTop() + child->getBaselineOfFirstLineBox();
+                    int ascent = child->firstLineBoxBaseline();
                     if (ascent == -1)
-                        ascent = child->marginTop() + child->height() + child->marginBottom();
+                        ascent = child->height() + child->marginBottom();
+                    ascent += child->marginTop();
                     childY += child->marginTop() + (maxAscent - ascent);
                     break;
                 }
@@ -480,10 +467,10 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
             placeChild(child, xPos, childY);
 
             if (child->isRenderBlock())
-                static_cast<RenderBlock*>(child)->addVisualOverflow(static_cast<RenderBlock*>(child)->floatRect());
+                toRenderBlock(child)->addVisualOverflow(toRenderBlock(child)->floatRect());
 
             m_overflowHeight = max(m_overflowHeight, childY + child->overflowHeight(false));
-            m_overflowTop = min(m_overflowTop, child->yPos() + child->overflowTop(false));
+            m_overflowTop = min(m_overflowTop, child->y() + child->overflowTop(false));
             
             xPos += child->width() + child->marginRight();
     
@@ -584,7 +571,9 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
     } while (haveFlex);
 
     m_flexingChildren = false;
-    
+
+    RenderBlock::finishDelayUpdateScrollInfo();
+
     if (remainingSpace > 0 && ((style()->direction() == LTR && style()->boxPack() != BSTART) ||
                                (style()->direction() == RTL && style()->boxPack() != BEND))) {
         // Children must be repositioned.
@@ -624,7 +613,7 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
                     remainingSpace -= (remainingSpace/totalChildren);
                     totalChildren--;
 
-                    placeChild(child, child->xPos()+offset, child->yPos());
+                    placeChild(child, child->x()+offset, child->y());
                     child = iterator.next();
                 }
             }
@@ -639,7 +628,7 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
                     child = iterator.next();
                     continue;
                 }
-                placeChild(child, child->xPos()+offset, child->yPos());
+                placeChild(child, child->x()+offset, child->y());
                 child = iterator.next();
             }
         }
@@ -651,28 +640,28 @@ void RenderFlexibleBox::layoutHorizontalBox(bool relayoutChildren)
     }
     
     if (child) {
-        m_overflowLeft = min(child->xPos() + child->overflowLeft(false), m_overflowLeft);
+        m_overflowLeft = min(child->x() + child->overflowLeft(false), m_overflowLeft);
 
-        RenderObject* lastChild = child;
+        RenderBox* lastChild = child;
         while ((child = iterator.next())) {
             if (!child->isPositioned())
                 lastChild = child;
         }
-        m_overflowWidth = max(lastChild->xPos() + lastChild->overflowWidth(false), m_overflowWidth);
+        m_overflowWidth = max(lastChild->x() + lastChild->overflowWidth(false), m_overflowWidth);
     }
     
     // So that the calcHeight in layoutBlock() knows to relayout positioned objects because of
     // a height change, we revert our height back to the intrinsic height before returning.
     if (heightSpecified)
-        m_height = oldHeight;
+        setHeight(oldHeight);
 }
 
 void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
 {
     int xPos = borderLeft() + paddingLeft();
     int yPos = borderTop() + paddingTop();
-    if( style()->direction() == RTL )
-        xPos = m_width - paddingRight() - borderRight();
+    if (style()->direction() == RTL)
+        xPos = width() - paddingRight() - borderRight();
     int toAdd = borderBottom() + paddingBottom() + horizontalScrollbarHeight();
     bool heightSpecified = false;
     int oldHeight = 0;
@@ -684,7 +673,7 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
     
     // The first walk over our kids is to find out if we have any flexible children.
     FlexBoxIterator iterator(this);
-    RenderObject *child = iterator.next();
+    RenderBox* child = iterator.next();
     while (child) {
         // Check to see if this child flexes.
         if (!child->isPositioned() && child->style()->boxFlex() > 0.0f) {
@@ -719,13 +708,13 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
                     
                     // Dirty all the positioned objects.
                     if (child->isRenderBlock()) {
-                        static_cast<RenderBlock*>(child)->markPositionedObjectsForLayout();
-                        static_cast<RenderBlock*>(child)->clearTruncation();
+                        toRenderBlock(child)->markPositionedObjectsForLayout();
+                        toRenderBlock(child)->clearTruncation();
                     }
                 }
                 child->layoutIfNeeded();
                 if (child->style()->height().isAuto() && child->isBlockFlow())
-                    maxLineCount = max(maxLineCount, static_cast<RenderBlock*>(child)->lineCount());
+                    maxLineCount = max(maxLineCount, toRenderBlock(child)->lineCount());
             }
             child = iterator.next();
         }
@@ -738,7 +727,7 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
                 if (child->isPositioned() || !child->style()->height().isAuto() || !child->isBlockFlow())
                     continue;
                 
-                RenderBlock* blockChild = static_cast<RenderBlock*>(child);
+                RenderBlock* blockChild = toRenderBlock(child);
                 int lineCount = blockChild->lineCount();
                 if (lineCount <= numVisibleLines)
                     continue;
@@ -767,9 +756,9 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
                 InlineBox* anchorBox = lastLine->lastChild();
                 if (!anchorBox)
                     continue;
-                if (!anchorBox->object()->element())
+                if (!anchorBox->renderer()->node())
                     continue;
-                if (!anchorBox->object()->element()->isLink())
+                if (!anchorBox->renderer()->node()->isLink())
                     continue;
                 
                 RootInlineBox* lastVisibleLine = blockChild->lineAtIndex(numVisibleLines-1);
@@ -777,7 +766,7 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
                     continue;
 
                 const UChar ellipsisAndSpace[2] = { horizontalEllipsis, ' ' };
-                static AtomicString ellipsisAndSpaceStr(ellipsisAndSpace, 2);
+                DEFINE_STATIC_LOCAL(AtomicString, ellipsisAndSpaceStr, (ellipsisAndSpace, 2));
 
                 const Font& font = style(numVisibleLines == 1)->font();
                 int ellipsisAndSpaceWidth = font.width(TextRun(ellipsisAndSpace, 2));
@@ -786,36 +775,42 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
                 int totalWidth = ellipsisAndSpaceWidth + anchorBox->width();
                 
                 // See if this width can be accommodated on the last visible line
-                RenderBlock* destBlock = static_cast<RenderBlock*>(lastVisibleLine->object());
-                RenderBlock* srcBlock = static_cast<RenderBlock*>(lastLine->object());
+                RenderBlock* destBlock = toRenderBlock(lastVisibleLine->renderer());
+                RenderBlock* srcBlock = toRenderBlock(lastLine->renderer());
                 
                 // FIXME: Directions of src/destBlock could be different from our direction and from one another.
                 if (srcBlock->style()->direction() != LTR)
                     continue;
                 if (destBlock->style()->direction() != LTR)
                     continue;
+                int ltr = true;
 
-                int blockEdge = destBlock->rightOffset(lastVisibleLine->yPos());
-                if (!lastVisibleLine->canAccommodateEllipsis(true, blockEdge, 
-                                                             lastVisibleLine->xPos() + lastVisibleLine->width(),
+                int blockRightEdge = destBlock->rightOffset(lastVisibleLine->y(), false);
+                int blockLeftEdge = destBlock->leftOffset(lastVisibleLine->y(), false);
+
+                int blockEdge = ltr ? blockRightEdge : blockLeftEdge;
+                if (!lastVisibleLine->canAccommodateEllipsis(ltr, blockEdge,
+                                                             lastVisibleLine->x() + lastVisibleLine->width(),
                                                              totalWidth))
                     continue;
 
                 // Let the truncation code kick in.
-                lastVisibleLine->placeEllipsis(ellipsisAndSpaceStr, true, blockEdge, totalWidth, anchorBox);
+                lastVisibleLine->placeEllipsis(ellipsisAndSpaceStr, ltr, blockLeftEdge, blockRightEdge, totalWidth, anchorBox);
                 destBlock->setHasMarkupTruncation(true);
             }
         }
     }
 
+    RenderBlock::startDelayUpdateScrollInfo();
+
     // We do 2 passes.  The first pass is simply to lay everyone out at
     // their preferred widths.  The second pass handles flexing the children.
     // Our first pass is done without flexing.  We simply lay the children
     // out within the box.
     do {
-        m_height = borderTop() + paddingTop();
-        int minHeight = m_height + toAdd;
-        m_overflowHeight = m_height;
+        setHeight(borderTop() + paddingTop());
+        int minHeight = height() + toAdd;
+        m_overflowHeight = height();
 
         child = iterator.first();
         while (child) {
@@ -826,14 +821,14 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
             if (child->isPositioned())
             {
                 child->containingBlock()->insertPositionedObject(child);
-                if (child->hasStaticX()) {
+                if (child->style()->hasStaticX()) {
                     if (style()->direction() == LTR)
-                        child->setStaticX(borderLeft()+paddingLeft());
+                        child->layer()->setStaticX(borderLeft()+paddingLeft());
                     else
-                        child->setStaticX(borderRight()+paddingRight());
+                        child->layer()->setStaticX(borderRight()+paddingRight());
                 }
-                if (child->hasStaticY())
-                    child->setStaticY(m_height);
+                if (child->style()->hasStaticY())
+                    child->layer()->setStaticY(height());
                 child = iterator.next();
                 continue;
             } 
@@ -842,7 +837,7 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
             child->calcVerticalMargins();
     
             // Add in the child's marginTop to our height.
-            m_height += child->marginTop();
+            setHeight(height() + child->marginTop());
     
             // Now do a layout.
             child->layoutIfNeeded();
@@ -869,39 +864,39 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
             }
     
             // Place the child.
-            placeChild(child, childX, m_height);
-            m_height += child->height() + child->marginBottom();
+            placeChild(child, childX, height());
+            setHeight(height() + child->height() + child->marginBottom());
     
             if (child->isRenderBlock())
-                static_cast<RenderBlock*>(child)->addVisualOverflow(static_cast<RenderBlock*>(child)->floatRect());
+                toRenderBlock(child)->addVisualOverflow(toRenderBlock(child)->floatRect());
 
             // See if this child has made our overflow need to grow.
-            m_overflowWidth = max(child->xPos() + child->overflowWidth(false), m_overflowWidth);
-            m_overflowLeft = min(child->xPos() + child->overflowLeft(false), m_overflowLeft);
+            m_overflowWidth = max(child->x() + child->overflowWidth(false), m_overflowWidth);
+            m_overflowLeft = min(child->x() + child->overflowLeft(false), m_overflowLeft);
             
             child = iterator.next();
         }
 
-        yPos = m_height;
+        yPos = height();
         
         if (!iterator.first() && hasLineIfEmpty())
-            m_height += lineHeight(true, true);
+            setHeight(height() + lineHeight(true, true));
     
-        m_height += toAdd;
+        setHeight(height() + toAdd);
 
         // Negative margins can cause our height to shrink below our minimal height (border/padding).
         // If this happens, ensure that the computed height is increased to the minimal height.
-        if (m_height < minHeight)
-            m_height = minHeight;
+        if (height() < minHeight)
+            setHeight(minHeight);
 
         // Always make sure our overflowheight is at least our height.
-        if (m_overflowHeight < m_height)
-            m_overflowHeight = m_height;
+        if (m_overflowHeight < height())
+            m_overflowHeight = height();
 
         // Now we have to calc our height, so we know how much space we have remaining.
-        oldHeight = m_height;
+        oldHeight = height();
         calcHeight();
-        if (oldHeight != m_height)
+        if (oldHeight != height())
             heightSpecified = true;
 
         remainingSpace = borderTop() + paddingTop() + contentHeight() - yPos;
@@ -996,6 +991,8 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
         }        
     } while (haveFlex);
 
+    RenderBlock::finishDelayUpdateScrollInfo();
+
     if (style()->boxPack() != BSTART && remainingSpace > 0) {
         // Children must be repositioned.
         int offset = 0;
@@ -1033,7 +1030,7 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
                     offset += remainingSpace/totalChildren;
                     remainingSpace -= (remainingSpace/totalChildren);
                     totalChildren--;
-                    placeChild(child, child->xPos(), child->yPos()+offset);
+                    placeChild(child, child->x(), child->y()+offset);
                     child = iterator.next();
                 }
             }
@@ -1048,7 +1045,7 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
                     child = iterator.next();
                     continue;
                 }
-                placeChild(child, child->xPos(), child->yPos()+offset);
+                placeChild(child, child->x(), child->y()+offset);
                 child = iterator.next();
             }
         }
@@ -1060,28 +1057,28 @@ void RenderFlexibleBox::layoutVerticalBox(bool relayoutChildren)
     }
     
     if (child) {
-        m_overflowTop = min(child->yPos() + child->overflowTop(false), m_overflowTop);
+        m_overflowTop = min(child->y() + child->overflowTop(false), m_overflowTop);
 
-        RenderObject* lastChild = child;
+        RenderBox* lastChild = child;
         while ((child = iterator.next())) {
             if (!child->isPositioned())
                 lastChild = child;
         }
-        m_overflowHeight = max(lastChild->yPos() + lastChild->overflowHeight(false), m_overflowHeight);
+        m_overflowHeight = max(lastChild->y() + lastChild->overflowHeight(false), m_overflowHeight);
     }
 
     // So that the calcHeight in layoutBlock() knows to relayout positioned objects because of
     // a height change, we revert our height back to the intrinsic height before returning.
     if (heightSpecified)
-        m_height = oldHeight; 
+        setHeight(oldHeight); 
 }
 
-void RenderFlexibleBox::placeChild(RenderObject* child, int x, int y)
+void RenderFlexibleBox::placeChild(RenderBox* child, int x, int y)
 {
-    IntRect oldRect(child->xPos(), child->yPos() , child->width(), child->height());
+    IntRect oldRect(child->x(), child->y() , child->width(), child->height());
 
     // Place the child.
-    child->setPos(x, y);
+    child->setLocation(x, y);
 
     // If the child moved, we have to repaint it as well as any floating/positioned
     // descendants.  An exception is if we need a layout.  In this case, we know we're going to
@@ -1090,7 +1087,7 @@ void RenderFlexibleBox::placeChild(RenderObject* child, int x, int y)
         child->repaintDuringLayoutIfMoved(oldRect);
 }
 
-int RenderFlexibleBox::allowedChildFlex(RenderObject* child, bool expanding, unsigned int group)
+int RenderFlexibleBox::allowedChildFlex(RenderBox* child, bool expanding, unsigned int group)
 {
     if (child->isPositioned() || child->style()->boxFlex() == 0.0f || child->style()->boxFlexGroup() != group)
         return 0;
