@@ -29,6 +29,9 @@
 #include "config.h"
 #include "Database.h"
 
+#include <wtf/StdLibExtras.h>
+
+#if ENABLE(DATABASE)
 #include "ChangeVersionWrapper.h"
 #include "CString.h"
 #include "DatabaseAuthorizer.h"
@@ -37,7 +40,6 @@
 #include "DatabaseTracker.h"
 #include "Document.h"
 #include "ExceptionCode.h"
-#include "FileSystem.h"
 #include "Frame.h"
 #include "InspectorController.h"
 #include "Logging.h"
@@ -45,41 +47,56 @@
 #include "Page.h"
 #include "OriginQuotaManager.h"
 #include "SQLiteDatabase.h"
+#include "SQLiteFileSystem.h"
 #include "SQLiteStatement.h"
 #include "SQLResultSet.h"
-#include <kjs/InitializeThreading.h>
 #include <wtf/MainThread.h>
+#endif
+
+#if USE(JSC)
+#include "JSDOMWindow.h"
+#include <runtime/InitializeThreading.h>
+#endif
 
 namespace WebCore {
 
-static Mutex& guidMutex()
-{
-    // FIXME: this is not a thread-safe way to initialize a shared global.
-    static Mutex mutex;
-    return mutex;
-}
-
-static HashMap<int, String>& guidToVersionMap()
-{
-    static HashMap<int, String> map;
-    return map;
-}
-
-static HashMap<int, HashSet<Database*>*>& guidToDatabaseMap()
-{
-    static HashMap<int, HashSet<Database*>*> map;
-    return map;
-}
+// If we sleep for more the 30 seconds while blocked on SQLITE_BUSY, give up.
+static const int maxSqliteBusyWaitTime = 30000;
 
 const String& Database::databaseInfoTableName()
 {
-    static String name = "__WebKitDatabaseInfoTable__";
+    DEFINE_STATIC_LOCAL(String, name, ("__WebKitDatabaseInfoTable__"));
     return name;
 }
 
+#if ENABLE(DATABASE)
+
+static Mutex& guidMutex()
+{
+    // Note: We don't have to use AtomicallyInitializedStatic here because
+    // this function is called once in the constructor on the main thread
+    // before any other threads that call this function are used.
+    DEFINE_STATIC_LOCAL(Mutex, mutex, ());
+    return mutex;
+}
+
+typedef HashMap<int, String> GuidVersionMap;
+static GuidVersionMap& guidToVersionMap()
+{
+    DEFINE_STATIC_LOCAL(GuidVersionMap, map, ());
+    return map;
+}
+
+typedef HashMap<int, HashSet<Database*>*> GuidDatabaseMap;
+static GuidDatabaseMap& guidToDatabaseMap()
+{
+    DEFINE_STATIC_LOCAL(GuidDatabaseMap, map, ());
+    return map;
+}
+
 static const String& databaseVersionKey()
 {
-    static String key = "WebKitDatabaseVersionKey";
+    DEFINE_STATIC_LOCAL(String, key, ("WebKitDatabaseVersionKey"));
     return key;
 }
 
@@ -93,7 +110,7 @@ PassRefPtr<Database> Database::openDatabase(Document* document, const String& na
         return 0;
     }
     
-    RefPtr<Database> database = new Database(document, name, expectedVersion);
+    RefPtr<Database> database = adoptRef(new Database(document, name, expectedVersion));
 
     if (!database->openAndVerifyVersion(e)) {
        LOG(StorageAPI, "Failed to open and verify version (expected %s) of database %s", expectedVersion.ascii().data(), database->databaseDebugName().ascii().data());
@@ -105,7 +122,7 @@ PassRefPtr<Database> Database::openDatabase(Document* document, const String& na
     document->setHasOpenDatabases();
 
     if (Page* page = document->frame()->page())
-        page->inspectorController()->didOpenDatabase(database.get(), document->domain(), name, expectedVersion);
+        page->inspectorController()->didOpenDatabase(database.get(), document->securityOrigin()->host(), name, expectedVersion);
 
     return database;
 }
@@ -118,6 +135,7 @@ Database::Database(Document* document, const String& name, const String& expecte
     , m_expectedVersion(expectedVersion)
     , m_deleted(false)
     , m_stopped(false)
+    , m_opened(false)
 {
     ASSERT(document);
     m_securityOrigin = document->securityOrigin();
@@ -125,7 +143,11 @@ Database::Database(Document* document, const String& name, const String& expecte
     if (m_name.isNull())
         m_name = "";
 
-    KJS::initializeThreading();
+#if USE(JSC)
+    JSC::initializeThreading();
+    // Database code violates the normal JSCore contract by calling jsUnprotect from a secondary thread, and thus needs additional locking.
+    JSDOMWindow::commonJSGlobalData()->heap.setGCProtectNeedsLocking();
+#endif
 
     m_guid = guidForOriginAndName(m_securityOrigin->toString(), name);
 
@@ -174,6 +196,8 @@ Database::~Database()
 
 bool Database::openAndVerifyVersion(ExceptionCode& e)
 {
+    if (!m_document->databaseThread())
+        return false;
     m_databaseAuthorizer = DatabaseAuthorizer::create();
 
     RefPtr<DatabaseOpenTask> task = DatabaseOpenTask::create(this);
@@ -213,7 +237,7 @@ static bool retrieveTextResultFromDatabase(SQLiteDatabase& db, const String& que
 
 bool Database::getVersionFromDatabase(String& version)
 {
-    static String getVersionQuery = "SELECT value FROM " + databaseInfoTableName() + " WHERE key = '" + databaseVersionKey() + "';";
+    DEFINE_STATIC_LOCAL(String, getVersionQuery, ("SELECT value FROM " + databaseInfoTableName() + " WHERE key = '" + databaseVersionKey() + "';"));
 
     m_databaseAuthorizer->disable();
 
@@ -249,7 +273,7 @@ static bool setTextValueInDatabase(SQLiteDatabase& db, const String& query, cons
 
 bool Database::setVersionInDatabase(const String& version)
 {
-    static String setVersionQuery = "INSERT INTO " + databaseInfoTableName() + " (key, value) VALUES ('" + databaseVersionKey() + "', ?);";
+    DEFINE_STATIC_LOCAL(String, setVersionQuery, ("INSERT INTO " + databaseInfoTableName() + " (key, value) VALUES ('" + databaseVersionKey() + "', ?);"));
 
     m_databaseAuthorizer->disable();
 
@@ -274,7 +298,7 @@ bool Database::versionMatchesExpected() const
 
 void Database::markAsDeletedAndClose()
 {
-    if (m_deleted)
+    if (m_deleted || !m_document->databaseThread())
         return;
 
     LOG(StorageAPI, "Marking %s (%p) as deleted", stringIdentifier().ascii().data(), this);
@@ -285,7 +309,7 @@ void Database::markAsDeletedAndClose()
         return;
     }
 
-    document()->databaseThread()->unscheduleDatabaseTasks(this);
+    m_document->databaseThread()->unscheduleDatabaseTasks(this);
 
     RefPtr<DatabaseCloseTask> task = DatabaseCloseTask::create(this);
 
@@ -296,7 +320,13 @@ void Database::markAsDeletedAndClose()
 
 void Database::close()
 {
-    m_sqliteDatabase.close();
+    if (m_opened) {
+        ASSERT(m_document->databaseThread());
+        ASSERT(currentThread() == document()->databaseThread()->getThreadID());
+        m_sqliteDatabase.close();
+        m_document->databaseThread()->recordDatabaseClosed(this);
+        m_opened = false;
+    }
 }
 
 void Database::stop()
@@ -318,10 +348,7 @@ void Database::stop()
 
 unsigned long long Database::databaseSize() const
 {
-    long long size;
-    if (!getFileSize(m_filename, size))
-        size = 0;
-    return size;
+    return SQLiteFileSystem::getDatabaseFileSize(m_filename);
 }
 
 unsigned long long Database::maximumSize() const
@@ -347,21 +374,30 @@ void Database::enableAuthorizer()
     m_databaseAuthorizer->enable();
 }
 
+void Database::setAuthorizerReadOnly()
+{
+    ASSERT(m_databaseAuthorizer);
+    m_databaseAuthorizer->setReadOnly();
+}
+
 static int guidForOriginAndName(const String& origin, const String& name)
 {
-    static int currentNewGUID = 1;
-    static Mutex stringIdentifierMutex;
-    static HashMap<String, int> stringIdentifierToGUIDMap;
-
     String stringID;
     if (origin.endsWith("/"))
         stringID = origin + name;
     else
         stringID = origin + "/" + name;
 
+    // Note: We don't have to use AtomicallyInitializedStatic here because
+    // this function is called once in the constructor on the main thread
+    // before any other threads that call this function are used.
+    DEFINE_STATIC_LOCAL(Mutex, stringIdentifierMutex, ());
     MutexLocker locker(stringIdentifierMutex);
+    typedef HashMap<String, int> IDGuidMap;
+    DEFINE_STATIC_LOCAL(IDGuidMap, stringIdentifierToGUIDMap, ());
     int guid = stringIdentifierToGUIDMap.get(stringID);
     if (!guid) {
+        static int currentNewGUID = 1;
         guid = currentNewGUID++;
         stringIdentifierToGUIDMap.set(stringID, guid);
     }
@@ -397,8 +433,13 @@ bool Database::performOpenAndVerify(ExceptionCode& e)
         return false;
     }
 
+    m_opened = true;
+    if (m_document->databaseThread())
+        m_document->databaseThread()->recordDatabaseOpen(this);
+
     ASSERT(m_databaseAuthorizer);
     m_sqliteDatabase.setAuthorizer(m_databaseAuthorizer);
+    m_sqliteDatabase.setBusyTimeout(maxSqliteBusyWaitTime);
 
     if (!m_sqliteDatabase.tableExists(databaseInfoTableName())) {
         if (!m_sqliteDatabase.executeCommand("CREATE TABLE " + databaseInfoTableName() + " (key TEXT NOT NULL ON CONFLICT FAIL UNIQUE ON CONFLICT REPLACE,value TEXT NOT NULL ON CONFLICT FAIL);")) {
@@ -408,18 +449,23 @@ bool Database::performOpenAndVerify(ExceptionCode& e)
         }
     }
 
-
     String currentVersion;
     {
         MutexLocker locker(guidMutex());
-        currentVersion = guidToVersionMap().get(m_guid);
 
-        if (currentVersion.isNull())
-            LOG(StorageAPI, "Current cached version for guid %i is null", m_guid);
-        else
+        // Note: It is not safe to put an empty string into the guidToVersionMap() map.
+        // That's because the map is cross-thread, but empty strings are per-thread.
+        // The copy() function makes a version of the string you can use on the current
+        // thread, but we need a string we can keep in a cross-thread data structure.
+        // FIXME: This is a quite-awkward restriction to have to program with.
+
+        GuidVersionMap::iterator entry = guidToVersionMap().find(m_guid);
+        if (entry != guidToVersionMap().end()) {
+            // Map null string to empty string (see comment above).
+            currentVersion = entry->second.isNull() ? String("") : entry->second;
             LOG(StorageAPI, "Current cached version for guid %i is %s", m_guid, currentVersion.ascii().data());
-
-        if (currentVersion.isNull()) {
+        } else {
+            LOG(StorageAPI, "No cached version for guid %i", m_guid);
             if (!getVersionFromDatabase(currentVersion)) {
                 LOG_ERROR("Failed to get current version from database %s", databaseDebugName().ascii().data());
                 e = INVALID_STATE_ERR;
@@ -434,11 +480,11 @@ bool Database::performOpenAndVerify(ExceptionCode& e)
                     e = INVALID_STATE_ERR;
                     return false;
                 }
-
                 currentVersion = m_expectedVersion;
             }
 
-            guidToVersionMap().set(m_guid, currentVersion.copy());
+            // Map empty string to null string (see comment above).
+            guidToVersionMap().set(m_guid, currentVersion.isEmpty() ? String() : currentVersion.copy());
         }
     }
 
@@ -554,6 +600,8 @@ void Database::deliverPendingCallback(void* context)
 
 Vector<String> Database::tableNames()
 {
+    if (!m_document->databaseThread())
+        return Vector<String>();
     RefPtr<DatabaseTableNamesTask> task = DatabaseTableNamesTask::create(this);
 
     task->lockForSynchronousScheduling();
@@ -579,4 +627,6 @@ String Database::stringIdentifier() const
     return m_name.copy();
 }
 
+#endif
+
 }
