@@ -29,18 +29,21 @@
 #include "config.h"
 #include "DatabaseTracker.h"
 
+#if ENABLE(DATABASE)
+
 #include "ChromeClient.h"
 #include "Database.h"
 #include "DatabaseTrackerClient.h"
 #include "Document.h"
-#include "FileSystem.h"
 #include "Logging.h"
 #include "OriginQuotaManager.h"
 #include "Page.h"
 #include "SecurityOrigin.h"
 #include "SecurityOriginHash.h"
+#include "SQLiteFileSystem.h"
 #include "SQLiteStatement.h"
 #include <wtf/MainThread.h>
+#include <wtf/StdLibExtras.h>
 
 using namespace std;
 
@@ -55,7 +58,7 @@ OriginQuotaManager& DatabaseTracker::originQuotaManager()
 
 DatabaseTracker& DatabaseTracker::tracker()
 {
-    static DatabaseTracker tracker;
+    DEFINE_STATIC_LOCAL(DatabaseTracker, tracker, ());
     return tracker;
 }
 
@@ -66,6 +69,7 @@ DatabaseTracker::DatabaseTracker()
     , m_thread(currentThread())
 #endif
 {
+    SQLiteFileSystem::registerSQLiteVFS();
 }
 
 void DatabaseTracker::setDatabaseDirectoryPath(const String& path)
@@ -84,9 +88,7 @@ const String& DatabaseTracker::databaseDirectoryPath() const
 String DatabaseTracker::trackerDatabasePath() const
 {
     ASSERT(currentThread() == m_thread);
-    if (m_databaseDirectoryPath.isEmpty())
-        return String();
-    return pathByAppendingComponent(m_databaseDirectoryPath, "Databases.db");
+    return SQLiteFileSystem::appendDatabaseFileNameToPath(m_databaseDirectoryPath, "Databases.db");
 }
 
 void DatabaseTracker::openTrackerDatabase(bool createIfDoesNotExist)
@@ -97,13 +99,9 @@ void DatabaseTracker::openTrackerDatabase(bool createIfDoesNotExist)
         return;
 
     String databasePath = trackerDatabasePath();
-    if (databasePath.isEmpty())
+    if (!SQLiteFileSystem::ensureDatabaseFileExists(databasePath, createIfDoesNotExist))
         return;
 
-    if (!createIfDoesNotExist && !fileExists(databasePath))
-        return;
-
-    makeAllDirectories(m_databaseDirectoryPath);
     if (!m_database.open(databasePath)) {
         // FIXME: What do do here?
         return;
@@ -187,9 +185,7 @@ bool DatabaseTracker::hasEntryForDatabase(SecurityOrigin* origin, const String&
 String DatabaseTracker::originPath(SecurityOrigin* origin) const
 {
     ASSERT(currentThread() == m_thread);
-    if (m_databaseDirectoryPath.isEmpty())
-        return String();
-    return pathByAppendingComponent(m_databaseDirectoryPath, origin->databaseIdentifier());
+    return SQLiteFileSystem::appendDatabaseFileNameToPath(m_databaseDirectoryPath, origin->databaseIdentifier());
 }
 
 String DatabaseTracker::fullPathForDatabase(SecurityOrigin* origin, const String& name, bool createIfNotExists)
@@ -203,7 +199,7 @@ String DatabaseTracker::fullPathForDatabase(SecurityOrigin* origin, const String
     String originPath = this->originPath(origin);
     
     // Make sure the path for this SecurityOrigin exists
-    if (createIfNotExists && !makeAllDirectories(originPath))
+    if (createIfNotExists && !SQLiteFileSystem::ensureDatabaseDirectoryExists(originPath))
         return String();
     
     // See if we have a path for this database yet
@@ -221,7 +217,7 @@ String DatabaseTracker::fullPathForDatabase(SecurityOrigin* origin, const String
     int result = statement.step();
 
     if (result == SQLResultRow)
-        return pathByAppendingComponent(originPath, statement.getColumnText(0));
+        return SQLiteFileSystem::appendDatabaseFileNameToPath(originPath, statement.getColumnText(0));
     if (!createIfNotExists)
         return String();
         
@@ -231,40 +227,20 @@ String DatabaseTracker::fullPathForDatabase(SecurityOrigin* origin, const String
     }
     statement.finalize();
     
-    SQLiteStatement sequenceStatement(m_database, "SELECT seq FROM sqlite_sequence WHERE name='Databases';");
-
-    // FIXME: More informative error handling here, even though these steps should never fail
-    if (sequenceStatement.prepare() != SQLResultOk)
-        return String();
-    result = sequenceStatement.step();
-
-    // This has a range of 2^63 and starts at 0 for every time a user resets Safari -
-    // I can't imagine it'd over overflow
-    int64_t seq = 0;
-    if (result == SQLResultRow) {
-        seq = sequenceStatement.getColumnInt64(0);
-    } else if (result != SQLResultDone)
-        return String();
-    sequenceStatement.finalize();
-
-    String filename;
-    do {
-        ++seq;
-        filename = pathByAppendingComponent(originPath, String::format("%016llx.db", seq));
-    } while (fileExists(filename));
-
-    if (!addDatabase(origin, name, String::format("%016llx.db", seq)))
+    String fileName = SQLiteFileSystem::getFileNameForNewDatabase(originPath, origin->databaseIdentifier(), name, &m_database);
+    if (!addDatabase(origin, name, fileName))
         return String();
 
     // If this origin's quota is being tracked (open handle to a database in this origin), add this new database
     // to the quota manager now
+    String fullFilePath = SQLiteFileSystem::appendDatabaseFileNameToPath(originPath, fileName);
     {
         Locker<OriginQuotaManager> locker(originQuotaManager());
         if (originQuotaManager().tracksOrigin(origin))
-            originQuotaManager().addDatabase(origin, name, filename);
+            originQuotaManager().addDatabase(origin, name, fullFilePath);
     }
     
-    return filename;
+    return fullFilePath;
 }
 
 void DatabaseTracker::populateOrigins()
@@ -420,8 +396,7 @@ unsigned long long DatabaseTracker::usageForDatabase(const String& name, Securit
     if (path.isEmpty())
         return 0;
         
-    long long size;
-    return getFileSize(path, size) ? size : 0;
+    return SQLiteFileSystem::getDatabaseFileSize(path);
 }
 
 void DatabaseTracker::addOpenDatabase(Database* database)
@@ -662,7 +637,7 @@ void DatabaseTracker::deleteOrigin(SecurityOrigin* origin)
         return;
     }
 
-    deleteEmptyDirectory(originPath(origin));
+    SQLiteFileSystem::deleteEmptyDatabaseDirectory(originPath(origin));
 
     RefPtr<SecurityOrigin> originPossiblyLastReference = origin;
     {
@@ -676,8 +651,8 @@ void DatabaseTracker::deleteOrigin(SecurityOrigin* origin)
         if (m_quotaMap->isEmpty()) {
             if (m_database.isOpen())
                 m_database.close();
-            deleteFile(trackerDatabasePath());
-            deleteEmptyDirectory(m_databaseDirectoryPath);
+           SQLiteFileSystem::deleteDatabaseFile(trackerDatabasePath());
+           SQLiteFileSystem::deleteEmptyDatabaseDirectory(m_databaseDirectoryPath);
         }
     }
 
@@ -760,7 +735,7 @@ bool DatabaseTracker::deleteDatabaseFile(SecurityOrigin* origin, const String& n
     for (unsigned i = 0; i < deletedDatabases.size(); ++i)
         deletedDatabases[i]->markAsDeletedAndClose();
 
-    return deleteFile(fullPath);
+    return SQLiteFileSystem::deleteDatabaseFile(fullPath);
 }
 
 void DatabaseTracker::setClient(DatabaseTrackerClient* client)
@@ -771,13 +746,15 @@ void DatabaseTracker::setClient(DatabaseTrackerClient* client)
 
 static Mutex& notificationMutex()
 {
-    static Mutex mutex;
+    DEFINE_STATIC_LOCAL(Mutex, mutex, ());
     return mutex;
 }
 
-static Vector<pair<SecurityOrigin*, String> >& notificationQueue()
+typedef Vector<pair<SecurityOrigin*, String> > NotificationQueue;
+
+static NotificationQueue& notificationQueue()
 {
-    static Vector<pair<SecurityOrigin*, String> > queue;
+    DEFINE_STATIC_LOCAL(NotificationQueue, queue, ());
     return queue;
 }
 
@@ -807,7 +784,7 @@ void DatabaseTracker::notifyDatabasesChanged(void*)
     // mechanism to include which tracker the notification goes out on as well.
     DatabaseTracker& theTracker(tracker());
 
-    Vector<pair<SecurityOrigin*, String> > notifications;
+    NotificationQueue notifications;
     {
         MutexLocker locker(notificationMutex());
 
@@ -825,3 +802,4 @@ void DatabaseTracker::notifyDatabasesChanged(void*)
 
 
 } // namespace WebCore
+#endif
