@@ -33,8 +33,7 @@
 #include "SVGTests.h"
 #include "SVGZoomAndPan.h"
 
-namespace WebCore
-{
+namespace WebCore {
     class SVGAngle;
     class SVGLength;
     class SVGTransform;
@@ -46,8 +45,7 @@ namespace WebCore
                           public SVGLangSpace,
                           public SVGExternalResourcesRequired,
                           public SVGFitToViewBox,
-                          public SVGZoomAndPan
-    {
+                          public SVGZoomAndPan {
     public:
         SVGSVGElement(const QualifiedName&, Document*);
         virtual ~SVGSVGElement();
@@ -111,16 +109,16 @@ namespace WebCore
         static SVGLength createSVGLength();
         static PassRefPtr<SVGAngle> createSVGAngle();
         static FloatPoint createSVGPoint();
-        static AffineTransform createSVGMatrix();
+        static TransformationMatrix createSVGMatrix();
         static FloatRect createSVGRect();
         static SVGTransform createSVGTransform();
-        static SVGTransform createSVGTransformFromMatrix(const AffineTransform&);
+        static SVGTransform createSVGTransformFromMatrix(const TransformationMatrix&);
 
         virtual void parseMappedAttribute(MappedAttribute*);
 
         // 'virtual SVGLocatable' functions
-        virtual AffineTransform getCTM() const;
-        virtual AffineTransform getScreenCTM() const;
+        virtual TransformationMatrix getCTM() const;
+        virtual TransformationMatrix getScreenCTM() const;
 
         virtual bool rendererIsNeeded(RenderStyle* style) { return StyledElement::rendererIsNeeded(style); }
         virtual RenderObject* createRenderer(RenderArena*, RenderStyle*);
@@ -130,7 +128,7 @@ namespace WebCore
 
         virtual void svgAttributeChanged(const QualifiedName&);
 
-        virtual AffineTransform viewBoxToViewTransform(float viewWidth, float viewHeight) const;
+        virtual TransformationMatrix viewBoxToViewTransform(float viewWidth, float viewHeight) const;
 
         void inheritViewAttributes(SVGViewElement*);
 
@@ -145,19 +143,13 @@ namespace WebCore
         bool isOutermostSVG() const;
 
     private:
-        void addSVGWindowEventListener(const AtomicString& eventType, const Attribute* attr);   
+        ANIMATED_PROPERTY_DECLARATIONS(SVGSVGElement, SVGNames::svgTagString, SVGNames::xAttrString, SVGLength, X, x)
+        ANIMATED_PROPERTY_DECLARATIONS(SVGSVGElement, SVGNames::svgTagString, SVGNames::yAttrString, SVGLength, Y, y)
+        ANIMATED_PROPERTY_DECLARATIONS(SVGSVGElement, SVGNames::svgTagString, SVGNames::widthAttrString, SVGLength, Width, width)
+        ANIMATED_PROPERTY_DECLARATIONS(SVGSVGElement, SVGNames::svgTagString, SVGNames::heightAttrString, SVGLength, Height, height)
 
-        ANIMATED_PROPERTY_FORWARD_DECLARATIONS(SVGExternalResourcesRequired, bool, ExternalResourcesRequired, externalResourcesRequired)
-        ANIMATED_PROPERTY_FORWARD_DECLARATIONS(SVGFitToViewBox, FloatRect, ViewBox, viewBox)
-        ANIMATED_PROPERTY_FORWARD_DECLARATIONS(SVGFitToViewBox, SVGPreserveAspectRatio*, PreserveAspectRatio, preserveAspectRatio)
-
-        ANIMATED_PROPERTY_DECLARATIONS(SVGSVGElement, SVGLength, SVGLength, X, x)
-        ANIMATED_PROPERTY_DECLARATIONS(SVGSVGElement, SVGLength, SVGLength, Y, y)
-        ANIMATED_PROPERTY_DECLARATIONS(SVGSVGElement, SVGLength, SVGLength, Width, width)
-        ANIMATED_PROPERTY_DECLARATIONS(SVGSVGElement, SVGLength, SVGLength, Height, height)
-        
-        virtual void willSaveToCache();
-        virtual void willRestoreFromCache();
+        virtual void documentWillBecomeInactive();
+        virtual void documentDidBecomeActive();
 
         bool m_useCurrentView;
         RefPtr<SMILTimeContainer> m_timeContainer;
@@ -171,5 +163,3 @@ namespace WebCore
 
 #endif // ENABLE(SVG)
 #endif
-
-// vim:ts=4:noet
