@@ -27,7 +27,6 @@
 #include "XSLTProcessor.h"
 
 #include "CString.h"
-#include "Cache.h"
 #include "Console.h"
 #include "DOMImplementation.h"
 #include "DOMWindow.h"
@@ -37,7 +36,7 @@
 #include "FrameLoader.h"
 #include "FrameView.h"
 #include "HTMLDocument.h"
-#include "HTMLTokenizer.h"
+#include "HTMLTokenizer.h" // for parseHTMLDocumentFragment
 #include "Page.h"
 #include "ResourceError.h"
 #include "ResourceHandle.h"
@@ -56,11 +55,10 @@
 #include <wtf/Assertions.h>
 #include <wtf/Platform.h>
 #include <wtf/Vector.h>
-#if PLATFORM(MAC)
-#include "SoftLinking.h"
-#endif
 
 #if PLATFORM(MAC)
+#include "SoftLinking.h"
+
 SOFT_LINK_LIBRARY(libxslt);
 SOFT_LINK(libxslt, xsltFreeStylesheet, void, (xsltStylesheetPtr sheet), (sheet))
 SOFT_LINK(libxslt, xsltFreeTransformContext, void, (xsltTransformContextPtr ctxt), (ctxt))
@@ -75,7 +73,7 @@ SOFT_LINK(libxslt, xsltNextImport, xsltStylesheetPtr, (xsltStylesheetPtr style),
 
 namespace WebCore {
 
-void XSLTProcessor::genericErrorFunc(void* userData, const char* msg, ...)
+void XSLTProcessor::genericErrorFunc(void*, const char*, ...)
 {
     // It would be nice to do something with this error message.
 }
@@ -101,14 +99,14 @@ void XSLTProcessor::parseErrorFunc(void* userData, xmlError* error)
             break;
     }
 
-    console->addMessage(XMLMessageSource, level, error->message, error->line, error->file);
+    console->addMessage(XMLMessageSource, LogMessageType, level, error->message, error->line, error->file);
 }
 
 // FIXME: There seems to be no way to control the ctxt pointer for loading here, thus we have globals.
 static XSLTProcessor* globalProcessor = 0;
 static DocLoader* globalDocLoader = 0;
 static xmlDocPtr docLoaderFunc(const xmlChar* uri,
-                                    xmlDictPtr dict,
+                                    xmlDictPtr,
                                     int options,
                                     void* ctxt,
                                     xsltLoadType type)
@@ -127,8 +125,15 @@ static xmlDocPtr docLoaderFunc(const xmlChar* uri,
 
             Vector<char> data;
 
-            if (globalDocLoader->frame() && globalDocLoader->doc()->securityOrigin()->canRequest(url))
-                globalDocLoader->frame()->loader()->loadResourceSynchronously(url, error, response, data);
+            bool requestAllowed = globalDocLoader->frame() && globalDocLoader->doc()->securityOrigin()->canRequest(url);
+            if (requestAllowed) {
+                globalDocLoader->frame()->loader()->loadResourceSynchronously(url, AllowStoredCredentials, error, response, data);
+                requestAllowed = globalDocLoader->doc()->securityOrigin()->canRequest(response.url());
+            }
+            if (!requestAllowed) {
+                data.clear();
+                globalDocLoader->printAccessDeniedMessage(url);
+            }
 
             Console* console = 0;
             if (Frame* frame = globalProcessor->xslStylesheet()->ownerDocument()->frame())
@@ -263,10 +268,8 @@ PassRefPtr<Document> XSLTProcessor::createDocumentFromSource(const String& sourc
         frame->setDocument(result);
     }
     
-    if (sourceIsDocument) {
+    if (sourceIsDocument)
         result->setURL(ownerDocument->url());
-        result->setBaseURL(ownerDocument->baseURL());
-    }
     result->open();
     
     RefPtr<TextResourceDecoder> decoder = TextResourceDecoder::create(sourceMIMEType);
@@ -280,7 +283,7 @@ PassRefPtr<Document> XSLTProcessor::createDocumentFromSource(const String& sourc
     return result.release();
 }
 
-static inline RefPtr<DocumentFragment> createFragmentFromSource(const String& sourceString, const String& sourceMIMEType, Node* sourceNode, Document* outputDoc)
+static inline RefPtr<DocumentFragment> createFragmentFromSource(const String& sourceString, const String& sourceMIMEType, Document* outputDoc)
 {
     RefPtr<DocumentFragment> fragment = new DocumentFragment(outputDoc);
     
@@ -302,7 +305,7 @@ static inline RefPtr<DocumentFragment> createFragmentFromSource(const String& so
 static xsltStylesheetPtr xsltStylesheetPointer(RefPtr<XSLStyleSheet>& cachedStylesheet, Node* stylesheetRootNode)
 {
     if (!cachedStylesheet && stylesheetRootNode) {
-        cachedStylesheet = new XSLStyleSheet(stylesheetRootNode->parent() ? stylesheetRootNode->parent() : stylesheetRootNode,
+        cachedStylesheet = XSLStyleSheet::create(stylesheetRootNode->parent() ? stylesheetRootNode->parent() : stylesheetRootNode,
             stylesheetRootNode->document()->url().string());
         cachedStylesheet->parseString(createMarkup(stylesheetRootNode));
     }
@@ -429,24 +432,24 @@ PassRefPtr<DocumentFragment> XSLTProcessor::transformToFragment(Node* sourceNode
     
     if (!transformToString(sourceNode, resultMIMEType, resultString, resultEncoding))
         return 0;
-    return createFragmentFromSource(resultString, resultMIMEType, sourceNode, outputDoc);
+    return createFragmentFromSource(resultString, resultMIMEType, outputDoc);
 }
 
-void XSLTProcessor::setParameter(const String& namespaceURI, const String& localName, const String& value)
+void XSLTProcessor::setParameter(const String& /*namespaceURI*/, const String& localName, const String& value)
 {
     // FIXME: namespace support?
     // should make a QualifiedName here but we'd have to expose the impl
     m_parameters.set(localName, value);
 }
 
-String XSLTProcessor::getParameter(const String& namespaceURI, const String& localName) const
+String XSLTProcessor::getParameter(const String& /*namespaceURI*/, const String& localName) const
 {
     // FIXME: namespace support?
     // should make a QualifiedName here but we'd have to expose the impl
     return m_parameters.get(localName);
 }
 
-void XSLTProcessor::removeParameter(const String& namespaceURI, const String& localName)
+void XSLTProcessor::removeParameter(const String& /*namespaceURI*/, const String& localName)
 {
     // FIXME: namespace support?
     m_parameters.remove(localName);
