@@ -4,6 +4,9 @@
  * Copyright (C) 2007 Apple Inc.
  * Copyright (C) 2008 Christian Dywan <christian@imendio.com>
  * Copyright (C) 2008 Collabora Ltd.
+ * Copyright (C) 2008 Nuanti Ltd.
+ * Copyright (C) 2009 Jan Alonzo <jmalonzo@gmail.com>
+ * Copyright (C) 2009 Gustavo Noronha Silva <gns@gnome.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -23,32 +26,56 @@
 
 #include "config.h"
 
+#include "webkitenumtypes.h"
 #include "webkitwebframe.h"
 #include "webkitwebview.h"
-#include "webkit-marshal.h"
+#include "webkitmarshal.h"
 #include "webkitprivate.h"
 
+#include "AnimationController.h"
 #include "CString.h"
+#include "DocumentLoader.h"
 #include "FrameLoader.h"
 #include "FrameLoaderClientGtk.h"
 #include "FrameTree.h"
 #include "FrameView.h"
+#include <glib/gi18n-lib.h>
+#include "GCController.h"
 #include "GraphicsContext.h"
 #include "HTMLFrameOwnerElement.h"
 #include "JSDOMWindow.h"
+#include "JSLock.h"
 #include "PrintContext.h"
 #include "RenderView.h"
+#include "RenderTreeAsText.h"
 #include "JSDOMBinding.h"
 #include "ScriptController.h"
+#include "SubstituteData.h"
 
 #include <JavaScriptCore/APICast.h>
 
+/**
+ * SECTION:webkitwebframe
+ * @short_description: The content of a #WebKitWebView
+ *
+ * A #WebKitWebView contains a main #WebKitWebFrame. A #WebKitWebFrame
+ * contains the content of one URI. The URI and name of the frame can
+ * be retrieved, the load status and progress can be observed using the
+ * signals and can be controlled using the methods of the #WebKitWebFrame.
+ * A #WebKitWebFrame can have any number of children and one child can
+ * be found by using #webkit_web_frame_find_frame.
+ *
+ * <informalexample><programlisting>
+ * /<!-- -->* Get the frame from the #WebKitWebView *<!-- -->/
+ * WebKitWebFrame *frame = webkit_web_view_get_main_frame (WEBKIT_WEB_VIEW(my_view));
+ * g_print("The URI of this frame is '%s'", webkit_web_frame_get_uri (frame));
+ * </programlisting></informalexample>
+ */
+
 using namespace WebKit;
 using namespace WebCore;
 using namespace std;
 
-extern "C" {
-
 enum {
     CLEARED,
     LOAD_COMMITTED,
@@ -63,7 +90,8 @@ enum {
 
     PROP_NAME,
     PROP_TITLE,
-    PROP_URI
+    PROP_URI,
+    PROP_LOAD_STATUS
 };
 
 static guint webkit_web_frame_signals[LAST_SIGNAL] = { 0, };
@@ -84,19 +112,33 @@ static void webkit_web_frame_get_property(GObject* object, guint prop_id, GValue
     case PROP_URI:
         g_value_set_string(value, webkit_web_frame_get_uri(frame));
         break;
+    case PROP_LOAD_STATUS:
+        g_value_set_enum(value, webkit_web_frame_get_load_status(frame));
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
     }
 }
 
+// Called from the FrameLoaderClient when it is destroyed. Normally
+// the unref in the FrameLoaderClient is destroying this object as
+// well but due reference counting a user might have added a reference...
+void webkit_web_frame_core_frame_gone(WebKitWebFrame* frame)
+{
+    ASSERT(WEBKIT_IS_WEB_FRAME(frame));
+    frame->priv->coreFrame = 0;
+}
+
 static void webkit_web_frame_finalize(GObject* object)
 {
     WebKitWebFrame* frame = WEBKIT_WEB_FRAME(object);
     WebKitWebFramePrivate* priv = frame->priv;
 
-    priv->coreFrame->loader()->cancelAndClear();
-    priv->coreFrame = 0;
+    if (priv->coreFrame) {
+        priv->coreFrame->loader()->cancelAndClear();
+        priv->coreFrame = 0;
+    }
 
     g_free(priv->name);
     g_free(priv->title);
@@ -114,7 +156,7 @@ static void webkit_web_frame_class_init(WebKitWebFrameClass* frameClass)
      */
     webkit_web_frame_signals[CLEARED] = g_signal_new("cleared",
             G_TYPE_FROM_CLASS(frameClass),
-            (GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
+            (GSignalFlags)G_SIGNAL_RUN_LAST,
             0,
             NULL,
             NULL,
@@ -123,16 +165,25 @@ static void webkit_web_frame_class_init(WebKitWebFrameClass* frameClass)
 
     webkit_web_frame_signals[LOAD_COMMITTED] = g_signal_new("load-committed",
             G_TYPE_FROM_CLASS(frameClass),
-            (GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
+            (GSignalFlags)G_SIGNAL_RUN_LAST,
             0,
             NULL,
             NULL,
             g_cclosure_marshal_VOID__VOID,
             G_TYPE_NONE, 0);
 
+    /**
+     * WebKitWebFrame::load-done
+     * @web_frame: the object on which the signal is emitted
+     *
+     * Emitted when frame loading is done.
+     *
+     * Deprecated: Use WebKitWebView::load-finished instead, and/or
+     * WebKitWebView::load-error to be notified of load errors
+     */
     webkit_web_frame_signals[LOAD_DONE] = g_signal_new("load-done",
             G_TYPE_FROM_CLASS(frameClass),
-            (GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
+            (GSignalFlags)G_SIGNAL_RUN_LAST,
             0,
             NULL,
             NULL,
@@ -142,7 +193,7 @@ static void webkit_web_frame_class_init(WebKitWebFrameClass* frameClass)
 
     webkit_web_frame_signals[TITLE_CHANGED] = g_signal_new("title-changed",
             G_TYPE_FROM_CLASS(frameClass),
-            (GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
+            (GSignalFlags)G_SIGNAL_RUN_LAST,
             0,
             NULL,
             NULL,
@@ -152,7 +203,7 @@ static void webkit_web_frame_class_init(WebKitWebFrameClass* frameClass)
 
     webkit_web_frame_signals[HOVERING_OVER_LINK] = g_signal_new("hovering-over-link",
             G_TYPE_FROM_CLASS(frameClass),
-            (GSignalFlags)(G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION),
+            (GSignalFlags)G_SIGNAL_RUN_LAST,
             0,
             NULL,
             NULL,
@@ -172,25 +223,40 @@ static void webkit_web_frame_class_init(WebKitWebFrameClass* frameClass)
      */
     g_object_class_install_property(objectClass, PROP_NAME,
                                     g_param_spec_string("name",
-                                                        "Name",
-                                                        "The name of the frame",
+                                                        _("Name"),
+                                                        _("The name of the frame"),
                                                         NULL,
                                                         WEBKIT_PARAM_READABLE));
 
     g_object_class_install_property(objectClass, PROP_TITLE,
                                     g_param_spec_string("title",
-                                                        "Title",
-                                                        "The document title of the frame",
+                                                        _("Title"),
+                                                        _("The document title of the frame"),
                                                         NULL,
                                                         WEBKIT_PARAM_READABLE));
 
     g_object_class_install_property(objectClass, PROP_URI,
                                     g_param_spec_string("uri",
-                                                        "URI",
-                                                        "The current URI of the contents displayed by the frame",
+                                                        _("URI"),
+                                                        _("The current URI of the contents displayed by the frame"),
                                                         NULL,
                                                         WEBKIT_PARAM_READABLE));
 
+    /**
+    * WebKitWebFrame:load-status:
+    *
+    * Determines the current status of the load.
+    *
+    * Since: 1.1.7
+    */
+    g_object_class_install_property(objectClass, PROP_LOAD_STATUS,
+                                    g_param_spec_enum("load-status",
+                                                      "Load Status",
+                                                      "Determines the current status of the load",
+                                                      WEBKIT_TYPE_LOAD_STATUS,
+                                                      WEBKIT_LOAD_FINISHED,
+                                                      WEBKIT_PARAM_READABLE));
+
     g_type_class_add_private(frameClass, sizeof(WebKitWebFramePrivate));
 }
 
@@ -209,6 +275,9 @@ static void webkit_web_frame_init(WebKitWebFrame* frame)
  * Creates a new #WebKitWebFrame initialized with a controlling #WebKitWebView.
  *
  * Returns: a new #WebKitWebFrame
+ *
+ * Deprecated: 1.0.2: #WebKitWebFrame can only be used to inspect existing
+ * frames.
  **/
 WebKitWebFrame* webkit_web_frame_new(WebKitWebView* webView)
 {
@@ -219,25 +288,26 @@ WebKitWebFrame* webkit_web_frame_new(WebKitWebView* webView)
     WebKitWebViewPrivate* viewPriv = WEBKIT_WEB_VIEW_GET_PRIVATE(webView);
 
     priv->webView = webView;
-    priv->client = new WebKit::FrameLoaderClient(frame);
-    priv->coreFrame = Frame::create(viewPriv->corePage, 0, priv->client).get();
+    WebKit::FrameLoaderClient* client = new WebKit::FrameLoaderClient(frame);
+    priv->coreFrame = Frame::create(viewPriv->corePage, 0, client).get();
     priv->coreFrame->init();
 
     return frame;
 }
 
-WebKitWebFrame* webkit_web_frame_init_with_web_view(WebKitWebView* webView, HTMLFrameOwnerElement* element)
+PassRefPtr<Frame> webkit_web_frame_init_with_web_view(WebKitWebView* webView, HTMLFrameOwnerElement* element)
 {
     WebKitWebFrame* frame = WEBKIT_WEB_FRAME(g_object_new(WEBKIT_TYPE_WEB_FRAME, NULL));
     WebKitWebFramePrivate* priv = frame->priv;
     WebKitWebViewPrivate* viewPriv = WEBKIT_WEB_VIEW_GET_PRIVATE(webView);
 
     priv->webView = webView;
-    priv->client = new WebKit::FrameLoaderClient(frame);
-    priv->coreFrame = Frame::create(viewPriv->corePage, element, priv->client).releaseRef();
-    priv->coreFrame->init();
+    WebKit::FrameLoaderClient* client = new WebKit::FrameLoaderClient(frame);
 
-    return frame;
+    RefPtr<Frame> coreFrame = Frame::create(viewPriv->corePage, element, client);
+    priv->coreFrame = coreFrame.get();
+
+    return coreFrame.release();
 }
 
 /**
@@ -248,7 +318,7 @@ WebKitWebFrame* webkit_web_frame_init_with_web_view(WebKitWebView* webView, HTML
  *
  * Return value: the title of @frame
  */
-const gchar* webkit_web_frame_get_title(WebKitWebFrame* frame)
+G_CONST_RETURN gchar* webkit_web_frame_get_title(WebKitWebFrame* frame)
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
 
@@ -264,7 +334,7 @@ const gchar* webkit_web_frame_get_title(WebKitWebFrame* frame)
  *
  * Return value: the URI of @frame
  */
-const gchar* webkit_web_frame_get_uri(WebKitWebFrame* frame)
+G_CONST_RETURN gchar* webkit_web_frame_get_uri(WebKitWebFrame* frame)
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
 
@@ -299,7 +369,7 @@ WebKitWebView* webkit_web_frame_get_web_view(WebKitWebFrame* frame)
  *
  * Return value: the name of @frame
  */
-const gchar* webkit_web_frame_get_name(WebKitWebFrame* frame)
+G_CONST_RETURN gchar* webkit_web_frame_get_name(WebKitWebFrame* frame)
 {
     g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
 
@@ -309,7 +379,8 @@ const gchar* webkit_web_frame_get_name(WebKitWebFrame* frame)
         return priv->name;
 
     Frame* coreFrame = core(frame);
-    ASSERT(coreFrame);
+    if (!coreFrame)
+        return "";
 
     String string = coreFrame->tree()->name();
     priv->name = g_strdup(string.utf8().data());
@@ -329,11 +400,98 @@ WebKitWebFrame* webkit_web_frame_get_parent(WebKitWebFrame* frame)
     g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
 
     Frame* coreFrame = core(frame);
-    ASSERT(coreFrame);
+    if (!coreFrame)
+        return NULL;
 
     return kit(coreFrame->tree()->parent());
 }
 
+/**
+ * webkit_web_frame_load_uri:
+ * @frame: a #WebKitWebFrame
+ * @uri: an URI string
+ *
+ * Requests loading of the specified URI string.
+ *
+ * Since: 1.1.1
+ */
+void webkit_web_frame_load_uri(WebKitWebFrame* frame, const gchar* uri)
+{
+    g_return_if_fail(WEBKIT_IS_WEB_FRAME(frame));
+    g_return_if_fail(uri);
+
+    Frame* coreFrame = core(frame);
+    if (!coreFrame)
+        return;
+
+    coreFrame->loader()->load(ResourceRequest(KURL(KURL(), String::fromUTF8(uri))), false);
+}
+
+static void webkit_web_frame_load_data(WebKitWebFrame* frame, const gchar* content, const gchar* mimeType, const gchar* encoding, const gchar* baseURL, const gchar* unreachableURL)
+{
+    Frame* coreFrame = core(frame);
+    ASSERT(coreFrame);
+
+    KURL baseKURL = baseURL ? KURL(KURL(), String::fromUTF8(baseURL)) : blankURL();
+
+    ResourceRequest request(baseKURL);
+
+    RefPtr<SharedBuffer> sharedBuffer = SharedBuffer::create(content, strlen(content));
+    SubstituteData substituteData(sharedBuffer.release(),
+                                  mimeType ? String::fromUTF8(mimeType) : String::fromUTF8("text/html"),
+                                  encoding ? String::fromUTF8(encoding) : String::fromUTF8("UTF-8"),
+                                  baseKURL,
+                                  KURL(KURL(), String::fromUTF8(unreachableURL)));
+
+    coreFrame->loader()->load(request, substituteData, false);
+}
+
+/**
+ * webkit_web_frame_load_string:
+ * @frame: a #WebKitWebFrame
+ * @content: an URI string
+ * @mime_type: the MIME type, or %NULL
+ * @encoding: the encoding, or %NULL
+ * @base_uri: the base URI for relative locations
+ *
+ * Requests loading of the given @content with the specified @mime_type,
+ * @encoding and @base_uri.
+ *
+ * If @mime_type is %NULL, "text/html" is assumed.
+ *
+ * If @encoding is %NULL, "UTF-8" is assumed.
+ *
+ * Since: 1.1.1
+ */
+void webkit_web_frame_load_string(WebKitWebFrame* frame, const gchar* content, const gchar* contentMimeType, const gchar* contentEncoding, const gchar* baseUri)
+{
+    g_return_if_fail(WEBKIT_IS_WEB_FRAME(frame));
+    g_return_if_fail(content);
+
+    webkit_web_frame_load_data(frame, content, contentMimeType, contentEncoding, baseUri, NULL);
+}
+
+/**
+ * webkit_web_frame_load_alternate_string:
+ * @frame: a #WebKitWebFrame
+ * @content: the alternate content to display as the main page of the @frame
+ * @base_url: the base URI for relative locations
+ * @unreachable_url: the URL for the alternate page content
+ *
+ * Request loading of an alternate content for a URL that is unreachable.
+ * Using this method will preserve the back-forward list. The URI passed in
+ * @base_url has to be an absolute URI.
+ *
+ * Since: 1.1.6
+ */
+void webkit_web_frame_load_alternate_string(WebKitWebFrame* frame, const gchar* content, const gchar* baseURL, const gchar* unreachableURL)
+{
+    g_return_if_fail(WEBKIT_IS_WEB_FRAME(frame));
+    g_return_if_fail(content);
+
+    webkit_web_frame_load_data(frame, content, NULL, NULL, baseURL, unreachableURL);
+}
+
 /**
  * webkit_web_frame_load_request:
  * @frame: a #WebKitWebFrame
@@ -351,11 +509,10 @@ void webkit_web_frame_load_request(WebKitWebFrame* frame, WebKitNetworkRequest*
     g_return_if_fail(WEBKIT_IS_NETWORK_REQUEST(request));
 
     Frame* coreFrame = core(frame);
-    ASSERT(coreFrame);
+    if (!coreFrame)
+        return;
 
-    // TODO: Use the ResourceRequest carried by WebKitNetworkRequest when it is implemented.
-    String string = String::fromUTF8(webkit_network_request_get_uri(request));
-    coreFrame->loader()->load(ResourceRequest(KURL(string)));
+    coreFrame->loader()->load(core(request), false);
 }
 
 /**
@@ -369,7 +526,8 @@ void webkit_web_frame_stop_loading(WebKitWebFrame* frame)
     g_return_if_fail(WEBKIT_IS_WEB_FRAME(frame));
 
     Frame* coreFrame = core(frame);
-    ASSERT(coreFrame);
+    if (!coreFrame)
+        return;
 
     coreFrame->loader()->stopAllLoaders();
 }
@@ -385,7 +543,8 @@ void webkit_web_frame_reload(WebKitWebFrame* frame)
     g_return_if_fail(WEBKIT_IS_WEB_FRAME(frame));
 
     Frame* coreFrame = core(frame);
-    ASSERT(coreFrame);
+    if (!coreFrame)
+        return;
 
     coreFrame->loader()->reload();
 }
@@ -413,7 +572,8 @@ WebKitWebFrame* webkit_web_frame_find_frame(WebKitWebFrame* frame, const gchar*
     g_return_val_if_fail(name, NULL);
 
     Frame* coreFrame = core(frame);
-    ASSERT(coreFrame);
+    if (!coreFrame)
+        return NULL;
 
     String nameString = String::fromUTF8(name);
     return kit(coreFrame->tree()->find(AtomicString(nameString)));
@@ -433,9 +593,10 @@ JSGlobalContextRef webkit_web_frame_get_global_context(WebKitWebFrame* frame)
     g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
 
     Frame* coreFrame = core(frame);
-    ASSERT(coreFrame);
+    if (!coreFrame)
+        return NULL;
 
-    return toGlobalRef(coreFrame->scriptProxy()->globalObject()->globalExec());
+    return toGlobalRef(coreFrame->script()->globalObject()->globalExec());
 }
 
 /**
@@ -449,7 +610,8 @@ GSList* webkit_web_frame_get_children(WebKitWebFrame* frame)
     g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
 
     Frame* coreFrame = core(frame);
-    ASSERT(coreFrame);
+    if (!coreFrame)
+        return NULL;
 
     GSList* children = NULL;
     for (Frame* child = coreFrame->tree()->firstChild(); child; child = child->tree()->nextSibling()) {
@@ -473,7 +635,8 @@ gchar* webkit_web_frame_get_inner_text(WebKitWebFrame* frame)
     g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
 
     Frame* coreFrame = core(frame);
-    ASSERT(coreFrame);
+    if (!coreFrame)
+        return g_strdup("");
 
     FrameView* view = coreFrame->view();
 
@@ -485,9 +648,30 @@ gchar* webkit_web_frame_get_inner_text(WebKitWebFrame* frame)
     return g_strdup(string.utf8().data());
 }
 
-#if GTK_CHECK_VERSION(2,10,0)
+/**
+ * webkit_web_frame_dump_render_tree:
+ * @frame: a #WebKitWebFrame
+ *
+ * Return value: Non-recursive render tree dump of @frame
+ */
+gchar* webkit_web_frame_dump_render_tree(WebKitWebFrame* frame)
+{
+    g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
 
-static void begin_print(GtkPrintOperation* op, GtkPrintContext* context, gpointer user_data)
+    Frame* coreFrame = core(frame);
+    if (!coreFrame)
+        return g_strdup("");
+
+    FrameView* view = coreFrame->view();
+
+    if (view && view->layoutPending())
+        view->layout();
+
+    String string = externalRepresentation(coreFrame->contentRenderer());
+    return g_strdup(string.utf8().data());
+}
+
+static void begin_print_callback(GtkPrintOperation* op, GtkPrintContext* context, gpointer user_data)
 {
     PrintContext* printContext = reinterpret_cast<PrintContext*>(user_data);
 
@@ -505,7 +689,7 @@ static void begin_print(GtkPrintOperation* op, GtkPrintContext* context, gpointe
     gtk_print_operation_set_n_pages(op, printContext->pageCount());
 }
 
-static void draw_page(GtkPrintOperation* op, GtkPrintContext* context, gint page_nr, gpointer user_data)
+static void draw_page_callback(GtkPrintOperation* op, GtkPrintContext* context, gint page_nr, gpointer user_data)
 {
     PrintContext* printContext = reinterpret_cast<PrintContext*>(user_data);
 
@@ -515,33 +699,72 @@ static void draw_page(GtkPrintOperation* op, GtkPrintContext* context, gint page
     printContext->spoolPage(ctx, page_nr, width);
 }
 
-static void end_print(GtkPrintOperation* op, GtkPrintContext* context, gpointer user_data)
+static void end_print_callback(GtkPrintOperation* op, GtkPrintContext* context, gpointer user_data)
 {
     PrintContext* printContext = reinterpret_cast<PrintContext*>(user_data);
     printContext->end();
 }
 
-void webkit_web_frame_print(WebKitWebFrame* frame)
+/**
+ * webkit_web_frame_print_full:
+ * @frame: a #WebKitWebFrame to be printed
+ * @operation: the #GtkPrintOperation to be carried
+ * @action: the #GtkPrintOperationAction to be performed
+ * @error: #GError for error return
+ *
+ * Prints the given #WebKitFrame, using the given #GtkPrintOperation
+ * and #GtkPrintOperationAction. This function wraps a call to
+ * gtk_print_operation_run() for printing the contents of the
+ * #WebKitWebFrame.
+ *
+ * Since: 1.1.5
+ */
+GtkPrintOperationResult webkit_web_frame_print_full(WebKitWebFrame* frame, GtkPrintOperation* operation, GtkPrintOperationAction action, GError** error)
 {
+    g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), GTK_PRINT_OPERATION_RESULT_ERROR);
+    g_return_val_if_fail(GTK_IS_PRINT_OPERATION(operation), GTK_PRINT_OPERATION_RESULT_ERROR);
+
     GtkWidget* topLevel = gtk_widget_get_toplevel(GTK_WIDGET(webkit_web_frame_get_web_view(frame)));
     if (!GTK_WIDGET_TOPLEVEL(topLevel))
         topLevel = NULL;
 
     Frame* coreFrame = core(frame);
-    ASSERT(coreFrame);
+    if (!coreFrame)
+        return GTK_PRINT_OPERATION_RESULT_ERROR;
 
     PrintContext printContext(coreFrame);
 
-    GtkPrintOperation* op = gtk_print_operation_new();
-    g_signal_connect(G_OBJECT(op), "begin-print", G_CALLBACK(begin_print), &printContext);
-    g_signal_connect(G_OBJECT(op), "draw-page", G_CALLBACK(draw_page), &printContext);
-    g_signal_connect(G_OBJECT(op), "end-print", G_CALLBACK(end_print), &printContext);
-    GError *error = NULL;
-    gtk_print_operation_run(op, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG, GTK_WINDOW(topLevel), &error);
-    g_object_unref(op);
+    g_signal_connect(operation, "begin-print", G_CALLBACK(begin_print_callback), &printContext);
+    g_signal_connect(operation, "draw-page", G_CALLBACK(draw_page_callback), &printContext);
+    g_signal_connect(operation, "end-print", G_CALLBACK(end_print_callback), &printContext);
+
+    return gtk_print_operation_run(operation, action, GTK_WINDOW(topLevel), error);
+}
+
+/**
+ * webkit_web_frame_print:
+ * @frame: a #WebKitWebFrame
+ *
+ * Prints the given #WebKitFrame, by presenting a print dialog to the
+ * user. If you need more control over the printing process, see
+ * webkit_web_frame_print_full().
+ *
+ * Since: 1.1.5
+ */
+void webkit_web_frame_print(WebKitWebFrame* frame)
+{
+    g_return_if_fail(WEBKIT_IS_WEB_FRAME(frame));
+
+    WebKitWebFramePrivate* priv = frame->priv;
+    GtkPrintOperation* operation = gtk_print_operation_new();
+    GError* error = 0;
+
+    webkit_web_frame_print_full(frame, operation, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG, &error);
+    g_object_unref(operation);
 
     if (error) {
-        GtkWidget* dialog = gtk_message_dialog_new(GTK_WINDOW(topLevel),
+        GtkWidget* window = gtk_widget_get_toplevel(GTK_WIDGET(priv->webView));
+        GtkWidget* dialog = gtk_message_dialog_new(GTK_WIDGET_TOPLEVEL(window) ? GTK_WINDOW(window) : 0,
                                                    GTK_DIALOG_DESTROY_WITH_PARENT,
                                                    GTK_MESSAGE_ERROR,
                                                    GTK_BUTTONS_CLOSE,
@@ -553,13 +776,82 @@ void webkit_web_frame_print(WebKitWebFrame* frame)
     }
 }
 
-#else
-
-void webkit_web_frame_print(WebKitWebFrame*)
+bool webkit_web_frame_pause_animation(WebKitWebFrame* frame, const gchar* name, double time, const gchar* element)
 {
-    g_warning("Printing support is not available in older versions of GTK+");
+    ASSERT(core(frame));
+    Element* coreElement = core(frame)->document()->getElementById(AtomicString(element));
+    if (!coreElement || !coreElement->renderer())
+        return false;
+    return core(frame)->animation()->pauseAnimationAtTime(coreElement->renderer(), AtomicString(name), time);
 }
 
-#endif
+bool webkit_web_frame_pause_transition(WebKitWebFrame* frame, const gchar* name, double time, const gchar* element)
+{
+    ASSERT(core(frame));
+    Element* coreElement = core(frame)->document()->getElementById(AtomicString(element));
+    if (!coreElement || !coreElement->renderer())
+        return false;
+    return core(frame)->animation()->pauseTransitionAtTime(coreElement->renderer(), AtomicString(name), time);
+}
+
+unsigned int webkit_web_frame_number_of_active_animations(WebKitWebFrame* frame)
+{
+    Frame* coreFrame = core(frame);
+    if (!coreFrame)
+        return 0;
+
+    AnimationController* controller = coreFrame->animation();
+    if (!controller)
+        return 0;
+
+    return controller->numberOfActiveAnimations();
+}
+
+gchar* webkit_web_frame_get_response_mime_type(WebKitWebFrame* frame)
+{
+    Frame* coreFrame = core(frame);
+    DocumentLoader* docLoader = coreFrame->loader()->documentLoader();
+    String mimeType = docLoader->responseMIMEType();
+    return g_strdup(mimeType.utf8().data());
+}
+
+/**
+ * webkit_web_frame_get_load_status:
+ * @frame: a #WebKitWebView
+ *
+ * Determines the current status of the load.
+ *
+ * Since: 1.1.7
+ */
+WebKitLoadStatus webkit_web_frame_get_load_status(WebKitWebFrame* frame)
+{
+    g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), WEBKIT_LOAD_FINISHED);
+
+    WebKitWebFramePrivate* priv = frame->priv;
+    return priv->loadStatus;
+}
+
+void webkit_web_frame_clear_main_frame_name(WebKitWebFrame* frame)
+{
+    g_return_if_fail(WEBKIT_IS_WEB_FRAME(frame));
+
+    core(frame)->tree()->clearName();
+}
+
+void webkit_gc_collect_javascript_objects()
+{
+    gcController().garbageCollectNow();
+}
+
+void webkit_gc_collect_javascript_objects_on_alternate_thread(gboolean waitUntilDone)
+{
+    gcController().garbageCollectOnAlternateThreadForDebugging(waitUntilDone);
+}
+
+gsize webkit_gc_count_javascript_objects()
+{
+    JSC::JSLock lock(JSC::SilenceAssertionsOnly);
+    return JSDOMWindow::commonJSGlobalData()->heap.objectCount();
 
 }
+
